#!/usr/bin/env node
// const P                    = require('bluebird');
const yargs = require('yargs');

const { patterns, urlPatternCollectionParts } = require('../../match');
const { fixDocByDoc, fixDocByDocAndParent } = require('../../commands/docFix');
const { genIteratorFuncs } = require('../../iterate');

const { genFirestore } = require('../helpers');

const { logChanges } = require('./helpers');

const firestore = genFirestore();
const { iterateCollection, iterateSubCollection, iterateCollectionGroup } = genIteratorFuncs(firestore);

const { nodryrun, _: args } = yargs
  .describe(`nodryrun`, `No dryrun`)
  .command('firestoreType', 'a firestore type or collectionGroup')
  .demandCommand(1, 'Please specify a firestore type or collectionGroup').argv;
const dryrun = !nodryrun;

const type = args[0];
if (!Object.keys(patterns).includes(type)) throw new Error(`no firestore match pattern found for: ${type}`);

const colGroup = patterns[type].colgroup;

const collectionParts = urlPatternCollectionParts(patterns[type]);

if (!colGroup && !(collectionParts.length === 1 || collectionParts.length === 2))
  throw new Error(`collection for ${type} is deeper than 2 collections`);

const isSubCol = collectionParts.length === 2;

const qOpts = {
  concurrency: 25,
  pageSize: 250,
};

if (colGroup) return iterateCollectionGroup(colGroup, (doc) => fixDocByDoc(doc, dryrun).then(logChanges), qOpts);

isSubCol
  ? iterateSubCollection(
      collectionParts[0],
      collectionParts[1],
      (doc2, doc1) => fixDocByDocAndParent(doc2, doc1, dryrun).then(logChanges),
      qOpts,
      qOpts
    )
  : iterateCollection(collectionParts[0], (doc) => fixDocByDoc(doc, dryrun).then(logChanges), qOpts);
