#!/usr/bin/env node
/* eslint-disable global-require */
/* eslint-disable import/no-dynamic-require */
const config = require('@hatsa/config')();
const path = require('path');
const yargs = require('yargs');
const { BigQuery } = require('@google-cloud/bigquery');

const genApiRequester = require('@hatsa/utils/api');
const { createLogger } = require('@hatsa/log');
const { getClient } = require('@hatsa/elasticsearch');
const { getAdmin } = require('../helpers');

const {
  nodryrun,
  debug,
  _: customFixer,
} = yargs
  .describe('nodryrun', 'No dryrun')
  .describe('debug', 'Debug')
  .command('customFixer', 'found in command/customFixers')
  .demandCommand(1, 'Please specify a firestore path').argv;

const dryrun = !nodryrun;
let fixer;

try {
  fixer = require(path.resolve(`./commands/customFixers/${customFixer}`));
} catch (e) {
  if (debug) console.error(e);
}

if (!fixer) throw new Error(`no custom fixer found for "${customFixer}"`);

const logger = createLogger();
const admin = getAdmin();
const firestore = admin.firestore();
const apiRequest = genApiRequester({
  admin,
  config,
});
const es = getClient({
  dsn: config.get('es_dsn'),
  prefix: config.get('google_cloud_project'),
  firestore,
  // logger,
});
const bigQuery = new BigQuery();

fixer({
  logger,
  admin,
  firestore,
  apiRequest,
  es,
  bigQuery,
  debug,
  nodryrun,
  dryrun,
  config,
})
  .then(() => logger.info('Custom migration finished.'))
  .catch((err) => logger.error('Migration error. ', err));
