const { createFastTraverser, createMigrator } = require('@firecode/admin');

const firecodeMigrator = async ({ logger, batchSize = 500, migratorSettings, dryrun }) => {
  const {
    query: colRef,
    runCloudFunctions,
    runWithFirecode: { op: operation, docDataModifier, docData, docOptions },
    maxConcurrentBatchCount = 10,
  } = migratorSettings;
  const traverser = createFastTraverser(colRef, {
    batchSize,
    maxConcurrentBatchCount,
    // maxDocCount: 10,
  });

  logger.info(
    `Migration started${
      dryrun ? '(dryrun)' : ''
    }, op=${operation}, batchSize=${batchSize}, maxConcurrentBatchCount=${maxConcurrentBatchCount}.`
  );

  if (dryrun) {
    const { docCount } = await traverser.traverse(async (_, batchIndex) => {
      logger.info(`Gonna process batch number ${batchIndex}.`);
    });
    logger.info(`Traversed ${docCount} documents`);
    return null;
  }
  const fastMigrator = createMigrator(traverser);
  const docModified = runCloudFunctions ? { ...docData } : { _migration: true, ...docData };

  const buildMigrator = () =>
    ({
      set: () => fastMigrator.set(docModified, docOptions),
      update: () => fastMigrator.update(docModified),
      updateWithDerivedData: () => {
        if (!docDataModifier) {
          throw new Error('updateWithDerivedData error, docDataModifier is not defined.');
        }
        return fastMigrator.updateWithDerivedData(docDataModifier);
      },
    }[operation]);

  const migrator = buildMigrator();
  if (!migrator) {
    throw new Error(`Firecode migrator operation(${operation}) is not supported or malformed`);
  }

  const { migratedDocCount } = await migrator();
  logger.info(`Migration finished. Migrated ${migratedDocCount} docs.`);
  return null;
};

module.exports = firecodeMigrator;
