const sentinels = require('../../../sentinel');

module.exports = (firestore) => {
  const { delete: deleteField } = sentinels(firestore);
  return {
    // A trick to filter out documents without a property.
    // Note: An orderBy() clause also filters for existence of the given field.
    // The result set will not include documents that do not contain the given field.
    // Docs - https://firebase.google.com/docs/firestore/query-data/order-limit-data#order_and_limit_data
    query: firestore.collection('offerers').orderBy('cloudinary'),
    runCloudFunctions: false,
    handle: (doc) => ({
      op: 'set',
      docRef: firestore.doc(`offerers/${doc.id}`),
      docData: {
        cloudinary: deleteField(),
      },
      docOptions: { merge: true },
    }),
  };
};
