const sentinels = require('../../../sentinel');

module.exports = (firestore) => {
  const { arrayUnion } = sentinels(firestore);
  return {
    query: firestore.collectionGroup('devices').select('platform'),
    id: (doc) => doc.ref.parent.parent.id,
    merge: (oldTask, newTask) => {
      if (oldTask._app && oldTask._app.includes(newTask.get('platform'))) return oldTask;

      if (oldTask.get('platform') === newTask.get('platform')) return oldTask;

      return {
        id: oldTask.id,
        _app: [oldTask.get('platform'), newTask.get('platform')],
        _profileId: oldTask.ref.parent.parent.id,
      };
    },
    handle: (task) => {
      const profileId = task.ref ? task.ref.parent.parent.id : task._profileId;
      const _app = task._app ? arrayUnion(...task._app) : arrayUnion(task.get('platform'));
      return {
        op: 'set',
        docRef: firestore.doc(`profiles/${profileId}`),
        docData: { _app },
        docOptions: { merge: true },
      };
    },
  };
};
