/**
 * Update board items likes counter
 *
 * @param firestore
 * @returns {*}
 */
module.exports = (firestore) => ({
  query: firestore.collectionGroup('likes'),
  runCloudFunctions: false,
  id: (doc) => doc.ref.id,
  filter: (doc) => doc.id.includes('item_'),
  merge: (task, newTask) => {
    const numLikes = task.numLikes || 1;
    const boardId = task.boardId || newTask.get('meta.boardId');
    const itemId = task.itemId || newTask.get('meta.itemId');
    return {
      id: task.id,
      numLikes: numLikes + 1,
      boardId,
      itemId,
    };
  },
  handle: (task) => {
    const numLikes = task.numLikes || 1;
    const boardId = task.boardId || task.get('meta.boardId');
    const itemId = task.itemId || task.get('meta.itemId');
    return {
      op: 'set',
      docRef: firestore.doc(`boards/${boardId}/items/${itemId}`),
      docData: { _num_likes: numLikes },
      docOptions: { merge: true },
    };
  },
});
