/**
 * Update profile boards counter
 */

const isDeleted = (board) => !!board.deleted_at;

const isPublic = (board) => !isDeleted(board) && !!board.public;

const hasPublicAssets = (board) =>
  !isDeleted(board) &&
  !!(isPublic(board) && board.poster && board.poster.assets && Object.keys(board.poster.assets).length);

module.exports = (firestore) => ({
  query: firestore.collection('boards').orderBy('created_by'),
  id: (doc) => doc.get('created_by'),
  merge: (oldTask, newTask) => {
    const oldTaskData = oldTask.ref ? oldTask.data() : {};
    const newTaskData = newTask.ref ? newTask.data() : {};
    const numBoards = oldTask.numBoards !== undefined ? oldTask.numBoards : +!isDeleted(oldTaskData);
    const numPublicBoards = (oldTask.numPublicBoards || +isPublic(oldTaskData)) + isPublic(newTaskData);
    const numPublicBoardsWithAssets =
      (oldTask.numPublicBoardsWithAssets || +hasPublicAssets(oldTaskData)) + hasPublicAssets(newTaskData);
    return {
      id: oldTask.id,
      numBoards: numBoards + !isDeleted(newTaskData),
      numPublicBoards,
      numPublicBoardsWithAssets,
      profileId: oldTask.profileId || oldTask.get('created_by'),
    };
  },
  handle: (task) => {
    const profileId = task.ref ? task.get('created_by') : task.profileId;
    const taskData = task.ref ? task.data() : {};
    const {
      numBoards = +!isDeleted(taskData),
      numPublicBoards = +isPublic(taskData),
      numPublicBoardsWithAssets = +hasPublicAssets(taskData),
    } = task;
    return {
      op: 'set',
      docRef: firestore.doc(`profiles/${profileId}`),
      docData: {
        _num_boards: numBoards,
        _num_boards_public: numPublicBoards,
        _num_public_boards_with_assets: numPublicBoardsWithAssets,
      },
      docOptions: { merge: true },
    };
  },
});
