/**
 * Update profile followers counter
 *
 * @param firestore
 * @returns {*}
 */
module.exports = (firestore) => ({
  query: firestore.collectionGroup('followers'),
  id: (doc) => doc.ref.parent.parent.id,
  merge: (oldTask) => {
    const numFollowers = oldTask.numFollowers || 1;
    return {
      id: oldTask.id,
      numFollowers: numFollowers + 1,
      profileId: oldTask.profileId || oldTask.ref.parent.parent.id,
    };
  },
  handle: (task) => {
    const profileId = task.ref ? task.ref.parent.parent.id : task.profileId;
    const numFollowers = task.numFollowers || 1;

    return {
      op: 'set',
      docRef: firestore.doc(`profiles/${profileId}`),
      docData: { _num_followers: numFollowers },
      docOptions: { merge: true },
    };
  },
});
