const P = require('bluebird');
// const identity = require('lodash/identity');
const { genIteratorFuncs } = require('../../iterate');

const fsQueries = [
  //TODO: migrate assets so all have a blacklist prop
  // {
  //     query: ['blacklist', '==', false],
  // },

  // {
  //     query: ['pending', '==', true],
  // },

  // {
  //     query: ['error', '>', 'Rate Limit Exceeded'],
  //     orderBy: 'error',
  // },

  {
    query: ['error', '>', 'Timeout waiting for parallel processing'],
    orderBy: 'error',
  },

  // {
  //     query: ['error', '>', ''],
  //     orderBy: 'error',
  //     filter: doc => {
  //         const error = doc.get('error');

  //         const arr = [
  //             'Resource not found -',
  //             'Empty file'
  //         ];

  //         if (arr.some(e => error.startsWith(e)))
  //             return false;

  //         console.log(error)

  //         return true;
  //     }
  // }
];

module.exports = ({ firestore, log, apiRequest, logger, nodryrun }) => {
  const refetchFromCld = (doc) =>
    P.resolve(
      apiRequest({
        method: 'post',
        url: `assets/s2s/${doc.id}/save`,
      })
    );

  const reUploadToCld = (doc) =>
    P.resolve(
      apiRequest({
        url: `assets/s2s/upload`,
        method: 'post',
        formData: {
          value: doc.get('url'),
        },
      })
    );

  const fixAsset = (doc, logger) => {
    const genErrContinue = (msg) => (res) => {
      let code, message;

      if (res.error.success === false) {
        message = res.error.error.message || 'Unknown error';
        code = res.error.error.code || 'Unknown status';
      } else {
        message = res.message;
        code = res.statusCode;
      }

      logger.warn(`${msg} (${code} ${message})`);

      return { success: false };
    };

    const genRunIfNotComplete =
      (func, ...args) =>
      ({ success }) => {
        if (success !== true) return func(...args);

        return { success };
      };

    return P.resolve({ success: false })
      .then(genRunIfNotComplete(refetchFromCld, doc))
      .tap(() => `${doc.id} refetched from cld and saved to firestore`)
      .catch(genErrContinue(`${doc.id} failed to refetch from cld`))
      .then(genRunIfNotComplete(reUploadToCld, doc))
      .tap(() => logger.info(`${doc.id} reuploaded to cld (${doc.get('url')})`))
      .catch(genErrContinue(`${doc.id} failed to reupload to cld (${doc.get('url')})`));
  };

  const { iterateCollection } = genIteratorFuncs(firestore);

  return P.each(fsQueries, (query) => {
    const handler = (doc) => {
      if (query.filter && !query.filter(doc)) return logger.info(`SKIPPED assets/${doc.id}`);

      if (nodryrun !== true) return logger.info(`DRYRUN assets/${doc.id} - ${doc.get('error') || ''}`);

      return fixAsset(doc, logger);
    };

    return iterateCollection('assets', handler, { where: query.query, orderBy: query.orderBy, pageSize: 250 }).then(
      (ids) => logger.info(`\n${ids.length} total`)
    );
  });
};
