const { DELETED_AT_PROP } = require('@hatsa/constants');
const { genBoardFuncs } = require('../../common/boards');
const { genUserFuncs } = require('../../common/users');
const { genIteratorFuncs } = require('../../iterate');

module.exports = async ({ firestore, nodryrun, debug, logger }) => {
  const { iterateCollection } = genIteratorFuncs(firestore);
  const { leaveParticipatingBoards } = genUserFuncs({ firestore });
  const { deleteUserBoards } = genBoardFuncs({ firestore });

  let totalUsers = 0;
  let totalBoards = 0;
  let totalParticipants = 0;
  const handler = async (profileSnap) => {
    totalUsers += 1;
    const userBoards = await deleteUserBoards(profileSnap.id, !nodryrun);
    const participatingBoards = await leaveParticipatingBoards(profileSnap, !nodryrun);

    logger.info(
      `${nodryrun ? '' : '[DRYRUN] '} [${profileSnap.id}] (displayName: ${profileSnap.get(
        'displayName'
      )}, ${DELETED_AT_PROP}: ${profileSnap.get(DELETED_AT_PROP).toDate().toISOString()}`
    );
    totalBoards += userBoards.length;
    totalParticipants += participatingBoards.length;

    if (debug) {
      const mapBoard = (b) => `${b.id} ${b.title}`;
      if (userBoards.length) {
        logger.info(`${nodryrun ? '' : '[DRYRUN] '} [${profileSnap.id}] userBoards`, userBoards.map(mapBoard));
      }
      if (participatingBoards.length) {
        logger.info(
          `${nodryrun ? '' : '[DRYRUN] '} [${profileSnap.id}] participatingBoards`,
          participatingBoards.map(mapBoard)
        );
      }
    }
  };

  return iterateCollection('profiles', handler, {
    orderBy: DELETED_AT_PROP,
    where: [DELETED_AT_PROP, '>', new Date('2000-01-01')],
    pageSize: 50,
    concurrency: 1,
  }).then(() =>
    logger.info('stats', {
      totalUsers,
      totalBoards,
      totalParticipants,
    })
  );
};
