const { DELETED_AT_PROP } = require('@hatsa/constants');
const { genIteratorFuncs } = require('../../iterate');

module.exports = async ({ es, firestore, dryrun, logger }) => {
  const { iterateCollection } = genIteratorFuncs(firestore);
  const startTime = process.hrtime.bigint();
  let totalBoards = 0;
  let totalItems = 0;

  const executionTime = () => `${Number(process.hrtime.bigint() - startTime) / 1e9} seconds`;

  const logPrefix = ({ boardId }) => `${dryrun ? '[DRYRUN] ' : ''} [${boardId}]`;

  const updateBoardItems = async ({ boardId, folder }) => {
    const query = {
      body: {
        query: {
          term: {
            board_id: boardId,
          },
        },
      },
    };

    if (dryrun) {
      const { hits } = await es.search('boarditems', query.body);
      logger.info(`[DRYRUN] [${boardId}] num of board items: ${hits.total.value}`);

      return hits.total.value;
    }

    const { updated } = await es.updateByQuery('boarditems', {
      script: {
        source: 'ctx._source.board_folder = params.folder',
        params: { folder },
      },
      query: query.body.query,
    });

    logger.info(`${logPrefix({ boardId })} num of updated board items: ${updated}`);
    return updated;
  };

  const handler = async (boardSnap) => {
    totalBoards += 1;

    const deletedAt = boardSnap.get(DELETED_AT_PROP);
    const folder = boardSnap.get('folder');

    logger.info(
      `${logPrefix({ boardId: boardSnap.id })} ` +
        `(` +
        `title: ${boardSnap.get('title')}, ` +
        `folder: ${folder}, ` +
        `${DELETED_AT_PROP}: ${deletedAt ? boardSnap.get(DELETED_AT_PROP).toDate().toISOString() : null}` +
        `)`
    );

    totalItems += await updateBoardItems({ boardId: boardSnap.id, folder });
  };

  return iterateCollection('boards', handler, {
    where: ['folder', 'in', ['deleted', 'archived']],
    pageSize: 100,
    concurrency: 1,
  }).then(() =>
    logger.info('stats', {
      totalBoards,
      totalItems,
      executionTime: executionTime(),
    })
  );
};
