/* eslint-disable vars-on-top */
const { batchUpdate } = require('../../batch');

const reviver = (key, value) =>
  value && typeof value === 'object' && value._seconds ? new Date(value._seconds * 1000) : value;

module.exports = async ({ firestore, nodryrun, logger }) => {
  const dryrun = !nodryrun;

  // eslint-disable-next-line no-use-before-define
  const data = offerersToFix
    .map(({ d }) => JSON.parse(d, reviver))
    .map((d) => {
      // eslint-disable-next-line
      delete d.aggregated_metadata;
      return d;
    });

  const batchCb = (chunk, batchNr, totalBatches) =>
    logger.info(`updated ${chunk.length} (batch ${batchNr}/${totalBatches})`);
  const dryrunCb = dryrun && batchCb;

  const docRefs = data.map((docData) => ({
    docRef: firestore.doc(`offerers/${docData.domain}`),
    docData,
  }));

  await batchUpdate(docRefs, {
    batchSize: 50,
    concurrency: 1,
    batchCb,
    dryrun,
    dryrunCb,
  });
};

// eslint-disable-next-line no-var
var offerersToFix = [
  // Result of BQ query:
  /*
    WITH
    touched_offerers as (
      SELECT DISTINCT(document_name) as id
      FROM `hatsa-prd.firestore.offerers_raw_changelog`
      WHERE timestamp >= timestamp('2022-08-16 12:10:16', 'Europe/Kiev')
    ),
    enriched_offerers as (
      SELECT
        ROW_NUMBER() OVER (PARTITION BY document_name ORDER BY timestamp DESC) AS sn,
        *
      FROM `hatsa-prd.firestore.offerers_raw_changelog`
      WHERE
        JSON_EXTRACT_SCALAR(data, '$.affiliateProgram') IS NOT NULL
        OR JSON_EXTRACT_SCALAR(data, '$.affiliation') IS NOT NULL
        OR JSON_EXTRACT_SCALAR(data, '$.dataMappings') IS NOT NULL
        OR JSON_EXTRACT_SCALAR(data, '$.strategy') IS NOT NULL
        OR JSON_EXTRACT_SCALAR(data, '$.parentId') IS NOT NULL
        OR JSON_EXTRACT_SCALAR(data, '$._temp_rating') IS NOT NULL
    )
    SELECT *
    FROM touched_offerers t1
    LEFT JOIN enriched_offerers t2 ON t1.id = t2.document_name
    WHERE
      t2.document_name IS NOT NULL
      AND sn = 1
  */
  // {
  //   id: 'projects/hatsa-prd/databases/(default)/documents/offerers/bol.com',
  //   sn: '1',
  //   timestamp: '2022-08-08T12:45:25.778Z',
  //   event_id: '57d17ad3-eb63-4ddf-bb11-c69e66563821-1',
  //   document_name: 'projects/hatsa-prd/databases/(default)/documents/offerers/bol.com',
  //   operation: 'UPDATE',
  //   data: '{"_temp_rating":5,"affiliateProgram":"/affiliates/bol/affiliate_programs/1025561","affiliation":{"identification":{"product":{"params":[{"siteLanguage":"(.*)"},{"productSlug":"(.*)"},{"productId":"(.*)"}],"regex":"/(.*)/p/(.*)/(.*)/"}},"last_updated":{"_seconds":1572625647,"_nanoseconds":987000000},"mutation":{"parameters":{},"type":"bol_redirect"},"useWWW":true},"aggregated_metadata":{"first":{"_seconds":1518121162,"_nanoseconds":0},"item_count":57285,"last":{"_seconds":1646058161,"_nanoseconds":0},"product_count":38543,"profile_count":8026},"blacklist":false,"categories":["tuin-buitenleven","sport-recreatie","schoenen","speelgoed","mooi-gezond","mode-accessoires","kleding","antiek-kunst","baby-peuter","boeken","dieren","elektronica-computers","doe-het-zelf","auto-motor-boot","eten-drinken","fietsen-scooters","games-film-muziek","hobby-vrijetijd","huis-inrichting","overige"],"checkFields":{"parentId":{"checked":true}},"dataMappings":[],"domain":"bol.com","domains":["bol.com"],"logo":"https://cdn.hatsa.com/hatsa-prd/offerers/bol.com/favicon","name":"bol.com","parentId":null,"strategy":{"name":"request","proxy":false,"version":"1.0.9"},"verification":{"code":"d6b89724cc09288cc3a698990d1afdfd64138f18","salt":"d2ddee28-5779-4ff7-9072-732eaac2e4fe","verified":false}}',
  // },
];
