const P = require('bluebird');

module.exports = ({ es, firestore, nodryrun, logger /* , debug */ }) => {
  const log = logger ? logger.info : console.log; //eslint-disable-line

  const handleDoc = ({ op, ref, prop, value, msg }) => {
    log(
      `${nodryrun !== true ? 'DRYRUN' : ''} ${ref.path} ${op || 'update'} ${prop || 'updated_at'}:${
        value || '(now)'
      } ${msg}`
    );

    if (nodryrun !== true) return undefined;

    if (op === 'delete') return ref.delete();

    return ref.update({ [prop || 'updated_at']: value || new Date() });
  };

  const esQuery = {
    body: {
      query: {
        term: {
          'assets_state.keyword': 'error',
        },
      },
    },
    // sort: [{ "post_date": { "created_at": "desc" } } ],
  };

  const itemFixHandler = (boardId) => (doc) => {
    const boardItemPath = `boards/${boardId}/items/${doc.id}`;

    if (doc.get('mode') === 'image' && doc.get('assets')) return undefined;

    // if ((doc.get('mode') === 'image') && !doc.get('assets')) {
    //     log(doc.ref.path, 'assetsless image item ');
    //     return;
    // }

    if (!(doc.get('urlref_id') || doc.get('urlref_ref') || doc.get('affiliate_product_id') || doc.get('url'))) {
      return handleDoc({
        op: 'delete',
        ref: firestore.doc(boardItemPath),
        msg: 'product item without url, urlref or affiliate_product_id',
      });
    }

    if (doc.get('url')) {
      return handleDoc({
        ref: firestore.doc(boardItemPath),
        msg: 'url needs scraping',
      });
    }

    if (!doc.get('urlref_id') || !doc.get('affiliate_product_id')) {
      log('no product, skipping');
      return undefined;
    }

    if (doc.get('urlref_id') && !doc.get('urlref')) {
      return handleDoc({
        ref: firestore.doc(boardItemPath),
        msg: 'urlref NOT POPULATED',
      });
    }

    // const urlref = doc.get('urlref');
    // if (!urlrefAssets) {
    //     return proms.push(handleDoc({
    //         ref: firestore.doc(`urlrefs/${doc.get('urlref_id')}`),
    //         prop: 'touch',
    //         msg: 'no assets NOT POPULATED at all'
    //     }));
    // }
    return undefined;
  };

  return es.getAllIds('boards', esQuery).then((boardIds) => {
    const getAndFixBoardItems = (boardId) =>
      firestore
        .collection(`boards/${boardId}/items`)
        .get()
        .then((ss) => ss.docs)
        .then((docs) => P.all(docs.map(itemFixHandler)));

    return P.map(boardIds, getAndFixBoardItems, { concurrency: 1 });
  });
};
