/* eslint-disable vars-on-top */
const P = require('bluebird');
const { offerersToFix } = require('./offerers-to-fix');

const reviver = (key, value) =>
  value && typeof value === 'object' && value._seconds ? new Date(value._seconds * 1000) : value;

module.exports = async ({ firestore, dryrun, logger }) => {
  const stats = {
    total: 0,
    touched: 0,
    ignored: 0,
  };
  const offerers = offerersToFix.map((d) => ({ ...d, affiliation: JSON.parse(d.affiliation, reviver) }));

  const handler = async ({ offererId, affiliation }) => {
    stats.total += 1;

    const offererRef = firestore.doc(`offerers/${offererId}`);
    const offererDoc = await offererRef.get();

    if (!offererDoc.exists) {
      if (dryrun) logger.info(`[${offererId}] offerer does not exist`);
      return;
    }

    const offererAffiliation = offererDoc.get('affiliation');

    if (offererAffiliation) {
      // do not touch affiliation if it's already exist
      stats.ignored += 1;
      return;
    }

    stats.touched += 1;

    logger.info(`[${offererId}] updating affiliation`);

    if (!dryrun) {
      offererRef.update({ affiliation });
    }
  };

  await P.map(offerers, handler, { concurrency: 10 });
  logger.info('stats', stats);
};
