const { batchUpdate } = require('../../batch');

const cs = console;

module.exports = ({ es, firestore, debug, logger, dryrun }) => {
  const log = logger ? logger.info : cs.log;

  const batchCb = function (chunk, chunkNr, chunkTotal) {
    log(`processed ${chunk.length} items (batch ${chunkNr}/${chunkTotal})`, debug && chunk.map((c) => c.docRef));
  };
  const dryrunCb = dryrun && batchCb;

  const touchAssets = (ids) =>
    batchUpdate(
      ids.map((id) => ({
        docRef: firestore.collection('assets').doc(id),
        docData: { touch: new Date() },
      })),
      { debug, dryrunCb, batchCb, batchSize: 50 }
    );

  //TODO: only urlrefs older than 1 day?
  const opts = {
    _source: ['assets_pending', 'amount_assets_pending'],
    body: {
      query: {
        range: {
          amount_assets_pending: {
            gte: 1,
          },
        },
      },
    },
  };

  const assets_to_touch = [];
  let total_urlrefs = 0;

  const func = (doc) => {
    doc._source.assets_pending.forEach((a) => {
      total_urlrefs += 1;
      if (!assets_to_touch.includes(a)) {
        assets_to_touch.push(a);
      }
    });
  };

  return es
    .scroll('urlrefs', opts, func)
    .then(() => {
      if (debug) {
        // log(JSON.stringify(assets_to_touch, null, 2));
        log(`total assets to touch ${assets_to_touch.length} (${total_urlrefs} urlrefs)`);
      }
    })
    .then(() => touchAssets(assets_to_touch));
};
