const P = require('bluebird');
const chunk = require('lodash/chunk');
const { genIteratorFuncs } = require('../../iterate');

module.exports = async ({ admin, dryrun, logger }) => {
  const db = admin.firestore();
  const startTime = process.hrtime.bigint();
  const getExecutionTime = () => `${Number(process.hrtime.bigint() - startTime) / 1e9} seconds`;
  const stats = {
    events_total: 0,
    events_without_participants: 0,
    profiles_touched: 0,
  };
  const { iterateCollection } = genIteratorFuncs(db);
  const profileIdsSet = new Set();

  const handleEvent = (eventSnap) => {
    stats.events_total += 1;
    const participantIds = Object.keys(eventSnap.get('participants') || {});

    if (participantIds.length === 0) {
      if (dryrun) logger.info(`[${eventSnap.id}] - "${eventSnap.get('title')}" (no participants) `);
      stats.events_without_participants += 1;
      return;
    }

    participantIds.forEach((id) => profileIdsSet.add(id));
  };

  await iterateCollection('events', handleEvent, {
    pageSize: 1000,
    concurrency: 100,
  });

  if (!dryrun) {
    const batches = chunk(
      Array.from(profileIdsSet.values()),
      500 // batch can handle 500 items at maximum
    ).map(async (profileIds) => {
      const batch = db.batch();
      profileIds.forEach((id) =>
        batch.update(db.doc(`/profiles/${id}`), {
          interests: admin.firestore.FieldValue.arrayUnion(`l:lootjestrekken`),
        })
      );
      return batch;
    });

    await P.map(batches, (batch) => batch.commit(), { concurrency: 5 });
  } else {
    Array.from(profileIdsSet.values()).forEach((id) => logger.info(`[${id}] Adding interest`));
  }

  logger.info('stats', {
    ...stats,
    profiles_touched: profileIdsSet.size,
    execution_time: getExecutionTime(),
  });
};
