const P = require('bluebird');
const { genCloudTasks } = require('@hatsa/cloud-tasks'); // eslint-disable-line import/no-extraneous-dependencies

module.exports = async ({ bigQuery, dryrun, config, logger }) => {
  const stats = {
    profiles_total: 0,
    cloud_tasks_created: 0,
  };
  const gcp = config.get('google_cloud_project') || 'hatsa-tst-01';
  const copernicaQueue = genCloudTasks({
    options: {
      project: gcp,
      queue: 'copernica-queue',
      url:
        gcp === 'hatsa-tst-01'
          ? 'https://copernica-handler-drj24aohpa-uc.a.run.app'
          : 'https://copernica-handler-3tdobyhptq-uc.a.run.app',
    },
    logger,
  });

  const fetchProfiles = async ({ page: _page, onPage }) => {
    const limit = 1000;
    const page = _page || 1;
    const offset = limit * page - limit;
    logger.info(`[${gcp}] Query profiles from big query ${limit}/${offset}`);

    const query = `
      SELECT document_id AS id,
      FROM \`${gcp}.firestore.profiles_raw_latest\`
      WHERE JSON_EXTRACT_SCALAR(DATA, '$.deleted_at._seconds') IS NULL
      ORDER BY document_id ASC LIMIT ${limit}
      OFFSET ${offset};
    `;

    const [rows] = await bigQuery.query({
      query,
      location: 'US',
    });

    stats.profiles_total += rows.length;
    await onPage(rows);

    if (rows.length === 0 || rows.length < limit) {
      return null;
    }

    return fetchProfiles({ page: page + 1, onPage });
  };

  const createCloudTask = async ({ id }) => {
    if (dryrun) {
      logger.info(`[${id}] creating cloud task`);
    } else {
      await copernicaQueue.createCloudTask({
        json: {
          command: 'upsertProfile',
          uid: id,
        },
      });
    }
    stats.cloud_tasks_created += 1;
  };

  await fetchProfiles({
    onPage: (profiles) => P.map(profiles, createCloudTask, { concurrency: 10 }),
  });

  logger.info('stats', stats);
};
