const { CREATED_BY_PROP } = require('@hatsa/constants');
const { genIteratorFuncs } = require('../../iterate');

module.exports = async ({ firestore, nodryrun, logger }) => {
  const { iterateCollectionGroup } = genIteratorFuncs(firestore);

  const handler = async (itemSnap) => {
    const itemId = itemSnap.id;
    const boardId = itemSnap.ref.parent.parent.id;
    const likesSnap = await firestore.collection(`boards/${boardId}/items/${itemId}/likes`).get();
    const numLikes = likesSnap.size;

    logger.info(`${nodryrun ? '' : '[DRYRUN] '} ${boardId} / ${itemId} likes = ${numLikes}`);

    if (!nodryrun) {
      return null;
    }

    await firestore.collection(`boards/${boardId}/items`).doc(itemId).set(
      {
        _num_likes: numLikes,
        _migration: true,
      },
      { merge: true }
    );

    return null;
  };

  return iterateCollectionGroup('items', handler, {
    orderBy: CREATED_BY_PROP,
    pageSize: 50,
    concurrency: 10,
  }).then(() => logger.info('Migration finished.'));
};
