const { batchUpdate } = require('../../batch');
const { genIteratorFuncs } = require('../../iterate');

const cs = console;

module.exports = ({ firestore, debug, dryrun, logger }) => {
  const log = logger ? logger.info : cs.log;
  const batchCb = function (chunk, chunkNr, chunkTotal) {
    log(`processed ${chunk.length} items (batch ${chunkNr}/${chunkTotal})`, debug && chunk.map((c) => c.docRef));
  };
  const dryrunCb = dryrun && batchCb;

  const touchUrlrefs = (ids) =>
    batchUpdate(
      ids.map((id) => ({
        docRef: firestore.collection('urlrefs').doc(id),
        docData: { touch: new Date() },
      })),
      { debug, batchSize: 50, batchCb, dryrunCb }
    );

  const { iterateCollection } = genIteratorFuncs(firestore);

  const handler = () => {};

  iterateCollection('urlrefs', handler, {
    orderBy: 'created_at',
    pageSize: 50,
    select: ['created_at'],
    where: ['created_at', '>', new Date('2019-07-15')],
  });
};
