const { DocChanges } = require('../../index');
const { removeIfExists } = require('../fixers');

const fixInvite = (activity) => {
  const user = activity.get('user');
  const invite = activity.get('invite');

  if (user && !invite) {
    return { invitee: user };
  }

  if (invite && invite.invitee) {
    delete invite.invitee.emailVerified;
    delete invite.invitee.email;
  }
  return invite;
};

const fixItem = (activity) => {
  const item = activity.get('item');
  if (!item.image) {
    item.image = '/assets/missing_boarditem_img.png';
  }
  return item;
};

const fixActivity = (activity, board, dryrun) => {
  const type = activity.get('type');
  const item = activity.get('item');

  switch (type) {
    case 'board':
      if (typeof item !== 'object') {
        return !dryrun
          ? activity.ref.delete().then(() => console.log(`deleted ${activity.id}`)) //eslint-disable-line
          : console.log(`dryrun: delete ${activity.id} due incorrect item type`); //eslint-disable-line
      }
      return new DocChanges({
        item: fixItem(activity),
      });

    case 'joined':
      return new DocChanges({
        user: removeIfExists(activity.get('user')),
        invite: fixInvite(activity),
      });
  }

  return new DocChanges({});
};

module.exports = fixActivity;
