const { DocChanges } = require('../../index');
const {
  deleteDoc,
  fixDate,
  fixPositiveFloat,
  removeFieldIfNotType,
  removeIfExists,
  nullifyIfEmpty,
} = require('../fixers');

const { fixUser } = require('../subtypes/user');
const { fixParticipants } = require('../subtypes/participants');

const fixBoard = (d, _, dryrun) => {
  if (!d.get('created_by')) {
    return !dryrun
      ? deleteDoc(d).then(() => console.log(`deleted ${d.id} ${d.get('title')}`)) //eslint-disable-line
      : console.log(`dryrun: delete ${d.id} due missing field: "created_by"`); //eslint-disable-line
  }

  return new DocChanges({
    created_at: fixDate(d.get('created_at')),
    updated_at: fixDate(d.get('updated_at')),
    tags: removeFieldIfNotType(d.get('tags'), 'array'),
    user: fixUser(d.get('user')),
    participants: fixParticipants(d.get('participants')),
    folder: nullifyIfEmpty(d.get('folder')),
    public: !!d.get('public'),
    // _temp_cat_label  : nullifyIfEmpty(d.get('_temp_cat_label')),
    budget: d.get('budget') ? fixPositiveFloat(d.get('budget')) : undefined,
    // budget_seperate  : ((d.get('budget_seperate') === false) ? 'total_budget' : d.get('budget_seperate')),

    raw: removeIfExists(d.get('raw')),
    readonly: removeIfExists(d.get('readonly')),
    amount: removeIfExists(d.get('amount')),
    __token: removeIfExists(d.get('__token')),
    guests: removeIfExists(d.get('guests')),
    likes: removeIfExists(d.get('likes')),
  });
};

module.exports = fixBoard;
