const { DocChanges } = require('../../index');
const { getDoc, removeIfExists } = require('../fixers');

const fixInvite = (notification) => {
  const user = notification.get('data.user');
  const invite = notification.get('data.invite');

  if (user && !invite) {
    return { invitee: user };
  }

  if (invite && invite.invitee) {
    delete invite.invitee.emailVerified;
    delete invite.invitee.email;
  }

  return invite;
};

const fixNotification = (notification, _profile, _dryrun) => {
  const type = notification.get('data.type');
  let docChanges = {};

  switch (type) {
    case 'joined':
      docChanges = Object.assign({}, docChanges, {
        'data.user': removeIfExists(notification.get('data.user')),
        'data.invite': fixInvite(notification),
      });
      break;
  }

  const boardId = notification.get('meta.board.id');
  if (!boardId) return new DocChanges(docChanges);

  return getDoc(`boards/${boardId}`).then((board) => {
    docChanges = Object.assign({}, docChanges, {
      'meta.board.created_at': notification.get('meta.board.created_at') || board.data().created_at,
    });
    return new DocChanges(docChanges);
  });
};

module.exports = fixNotification;
