const { DocChanges } = require('../../index');
const { removeIfExists, fixDate, boolCast, deleteField, removeIfEmpty } = require('../fixers');

const fixGender = (g) => (['male', 'female'].includes(g) ? g : null);

const fixAssets = (assets) => {
  if (!assets) return undefined;

  const assetIds = Object.keys(assets || {});

  if (!assetIds.length) return deleteField;

  return assetIds.reduce((acc, assetId) => {
    acc[assetId].pending = false;
    return acc;
  }, assets);
};

const fixPhotoUrl = (d) => {
  // if (!d.get('photoURL'))
  //     return (d.get('gender') === 'female') ? '/assets/avatars/5.png' : '/assets/avatars/1.png';

  if (d.get('photoURL') && d.get('photoURL').startsWith('/assets/user.jpg')) return 'user.png';

  return d.get('photoURL') || null;
};

const fixProfiles = (d) => {
  const defFixes = {
    role: removeIfExists(d.get('role')),
    admin: removeIfExists(d.get('admin')),
    email: removeIfExists(d.get('email')),
    emailVerified: removeIfExists(d.get('emailVerified')),
    isAnonymous: removeIfExists(d.get('isAnonymous')),
    lastLoginAt: removeIfExists(d.get('lastLoginAt')),
    createdAt: removeIfExists(d.get('createdAt')),
    phoneNumber: removeIfExists(d.get('phoneNumber')),
    providerData: removeIfExists(d.get('providerData')),
    avatarUrl: removeIfExists(d.get('avatarUrl')),
    presence: removeIfExists(d.get('presence')),
    actions: removeIfExists(d.get('actions')),
    day: removeIfExists(d.get('day')),
    month: removeIfExists(d.get('month')),
    year: removeIfExists(d.get('year')),
    // locale:         removeIfEmpty(d.get('locale'))

    // _optins :       removeIfExists(d.get('_optins')),
  };

  let fixes;

  // console.log(!!d.get('birthdate'), d.get('birthdate'))

  if (d.get('deleted_at')) {
    fixes = {
      created_at: fixDate(d.get('created_at')),
      // updated_at       : fixDate(d.get('updated_at')),
      deleted_at: fixDate(d.get('deleted_at')),
      birthdate: removeIfExists(d.get('birthdate')),
      _temp_rating: removeIfExists(d.get('_temp_rating')),
      _num_following: removeIfExists(d.get('_num_following')),
      _num_followers: removeIfExists(d.get('_num_followers')),
      _slug: removeIfExists(d.get('_slug')),
      gender: removeIfExists(d.get('gender')),
    };
  } else {
    fixes = {
      created_at: fixDate(d.get('created_at')),
      updated_at: fixDate(d.get('updated_at')),
      public: boolCast(d.get('public')),
      gender: fixGender(d.get('gender')),
      photoURL: fixPhotoUrl(d),
      displayName: d.get('displayName') ? d.get('displayName') : 'Gebruiker',
      assets: fixAssets(d.get('assets')),
      _referral: removeIfEmpty(d.get('_referral')),
      // _referral        : Object.keys((d.get('_referral') || {})).length ? d.get('_referral') : deleteField
    };
  }

  return new DocChanges(Object.assign(defFixes, fixes));
};

module.exports = fixProfiles;
