const isEmpty = require('lodash/isEmpty');

const { DocChanges } = require('../../index');
const { removeIfExists } = require('../fixers');

const fixProduct = (d) => {
  const product = d.get('product');

  if (!product || typeof product !== 'object' || !Object.keys(product).length) return null;

  if (product.deliveryTime) delete product.deliveryTime;

  return product;
};

const fixUrls = (d) => {
  const urls = d.get('urls');

  const PUP_FUCK_UP = 'http://127.0.0.1:2004/?url=';
  const fixPupFuckUp = (url) => (url.startsWith(PUP_FUCK_UP) ? url.split(PUP_FUCK_UP)[1] : url);
  const fixWWW = (url) => (url.startsWith('www.') ? `https://${url}` : url);

  ['canonical', 'normalized'].forEach((dt) => {
    urls[dt] = fixWWW(fixPupFuckUp(urls[dt]));
  });

  return urls;
};

const fixDomains = (d) => {
  const domains = d.get('domains');

  if (!domains || !Array.isArray(domains) || !domains.length) return;

  return domains.filter((d) => d !== '127.0.0.1');
};

const fixUrlref = (d, _, dryrun) => {
  if (isEmpty(d.get('urls')))
    return dryrun === false ? d.ref.delete() : console.log('Doc will be removed because no urls found'); //eslint-disable-line

  const fixes = {};

  return (
    Promise.resolve()
      // .then(() => !d.get('refcount') && d.ref.collection('refs').get().then(refs => refs.size).then(refcount => fixes.refcount = refcount))
      .then(() =>
        Object.assign(fixes, {
          // created_at      : fixCreatedAt(d),
          // updated_at      : fixUpdatedAt(d),
          product: fixProduct(d),
          urls: fixUrls(d),
          domains: fixDomains(d),

          parsed_by: removeIfExists(d.get('parsed_by')),
          parsed_at: removeIfExists(d.get('parsed_at')),
          deliveryTime: removeIfExists(d.get('deliveryTime')),
          created_by: typeof d.get('created_by') !== 'string' ? d.get('parsed_by') : undefined,
        })
      )
      .then(() => new DocChanges(fixes))
  );
};

module.exports = fixUrlref;
