// eslint-disable-next-line import/no-extraneous-dependencies
const admin = require('firebase-admin');
const isEmpty = require('lodash/isEmpty');
const isEqual = require('lodash/isEqual');

const timestamp = admin.firestore.FieldValue.serverTimestamp();
const deleteField = admin.firestore.FieldValue.delete();

const fixDate = (d) => (d && typeof d.toDate === 'function' ? d : timestamp);

const dateToInt = (d) => new Date(d).getTime();
const dateIsOut = (d1, d2, threshold = 3000) => Math.abs(dateToInt(d1) - dateToInt(d2)) > threshold;

const createdAtIsOut = (d) => dateIsOut(d.createTime, d.get('created_at'));
const updatedAtIsOut = (d) => dateIsOut(d.updateTime, d.get('updated_at'));

const fixCreatedAt = (d) =>
  !(d.get('created_at') instanceof Date) || createdAtIsOut(d) ? new Date(d.createTime) : undefined;
const fixUpdatedAt = (d) =>
  !(d.get('updated_at') instanceof Date) || updatedAtIsOut(d) ? new Date(d.updateTime) : undefined;

const fixPositiveFloat = (d, def = 0) => Math.abs(parseFloat(d) || def);
const fixPositiveInteger = (d, def = 0) => Math.abs(parseInt(d) || def);

const deleteDoc = (d) => d.ref.delete();
// const getDoc = path => firestore.doc(path).get();

const removeFieldIfNotType = (d, type) => {
  if (['undefined', type].indexOf(typeof d) !== -1) return d;

  if (Array.isArray(d) && type === 'array') return undefined;

  return deleteField;
};

const removeFieldIfNotValue = (d, v) => (isEqual(d, v) ? undefined : deleteField);
const removeFieldIfValue = (d, v) => (isEqual(d, v) ? deleteField : undefined);

const removeIfExists = (d) => {
  if (typeof d !== 'undefined') return deleteField;
  return undefined;
};

const nullifyIfEmpty = (data) => {
  if (typeof data === 'undefined') return null;

  if (isEmpty(data)) return null;
  return undefined;
};

const removeIfEmpty = (data) => {
  if (typeof data === 'undefined') return undefined;

  if (isEmpty(data)) return deleteField;

  return undefined;
};

const boolCast = (data) => !!data;

module.exports = {
  fixDate,
  fixCreatedAt,
  fixUpdatedAt,

  fixPositiveFloat,
  fixPositiveInteger,

  // getDoc,
  deleteDoc,
  removeFieldIfNotType,
  removeFieldIfNotValue,
  removeFieldIfValue,
  removeIfExists,
  nullifyIfEmpty,
  removeIfEmpty,
  boolCast,

  deleteField,
};
