const map = require('lodash/map');
const diffpatch = require('jsondiffpatch');

const { isNonPojo, getOCName } = require('@hatsa/utils/object');

const pwt = (d) => {
  if (d === null) return 'null:null';

  if (d instanceof Date) return `Date:${d.toJSON()}`;

  if (Array.isArray(d)) return `Array:[${d.join(', ')}]`;

  if (isNonPojo(d)) return `object:${getOCName(d)}`;

  const v = typeof d === 'object' ? `object(${Object.keys(d).length} keys)` : JSON.stringify(d);

  return `${typeof d}:${v}`;
};

const renderChanges = (doc, changes) =>
  map(changes || {}, (v, k) => {
    const from = doc.get(k);
    const to = v;

    const isFirestoreValue = v && v.path && v.firestore; //firestore reference value

    const diff = diffpatch.diff(from, isFirestoreValue ? {} : to);
    const change = diffpatch.console.format(diff);

    return `PROP ${k} FROM ${pwt(from)} TO ${pwt(to)} ${change}`;
  });

module.exports = {
  renderChanges,
};
