const { fixDocByDoc } = require('./docFix');
const { genValidator } = require('../validate');
const { genIteratorFuncs } = require('../iterate');

const fixCollectedErrors =
  (firestore) =>
  (type, opts = {}) => {
    const { dryrun } = opts;

    const { iterateCollection } = genIteratorFuncs(firestore);
    const { validateDoc } = genValidator();

    const c = console;
    const log = (m, ...a) => c.log(`${dryrun ? 'DRYRUN: ' : ''}${m}`, ...a);

    const handler = (seDoc) =>
      seDoc
        .get('docRef')
        .get()
        .then((doc) => {
          if (!doc.exists) {
            log(`${doc.ref.path} seems to be deleted ${dryrun ? '' : ', removing from schema-errors'}`); //eslint-disable-line
            return /*!dryrun && */ seDoc.ref.delete();
          }

          if (validateDoc(doc, { debug: false, doThrow: false })) {
            log(`${doc.ref.path} seems to be fixed already ${dryrun ? '' : ', removing from schema-errors'}`); //eslint-disable-line
            return /*!dryrun && */ seDoc.ref.delete();
          }

          return fixDocByDoc(doc, dryrun).then((res) => {
            if (!res.changes.length)
              return log(
                `${doc.ref.path} no fixers found, still has schema error with hash: ${seDoc.get('errorHash')}`
              ); //eslint-disable-line

            res.changes.forEach((c) => log(`${doc.ref.path}`, c)); //eslint-disable-line
          });
        });

    if (type) opts.where = ['type', '==', type];

    return iterateCollection('schema-errors', handler, opts);
  };

module.exports = fixCollectedErrors;
