const log = console.log; //eslint-disable-line

const cloneWith = require('lodash/cloneDeepWith');
const map = require('lodash/map');
const get = require('lodash/get');
const identity = require('lodash/identity');
const padEnd = require('lodash/padEnd');

const pad = (d, p) => (p ? `${padEnd(`${d}`.substr(0, p), p)}` : d);

const { genIteratorFuncs } = require('../../iterate');
const { refToPath } = require('../../index');

// Make doc firesore document instance printable (replace Timestamps and References)
const fsd = (d) =>
  cloneWith(d.data(), (v) => {
    if (v && v.toDate) return v.toDate();

    if (v && v.get) return `Reference: ${refToPath(v)}`;

    return undefined;
  });

// Print object as table based on select array data
const tline = (o, qselect) => {
  if (typeof o !== 'object') return '';

  const select = qselect || Object.keys(o);
  const s = select.reduce((acc, val) => {
    if (val.path) {
      acc[val.path] = {
        pad: typeof val.pad === 'number' ? val.pad : 24,
        render: typeof val.render === 'function' ? val.render : identity,
      };
    } else {
      acc[val] = {
        pad: 24,
        render: identity,
      };
    }

    return acc;
  }, {});

  return map(s, (v, k) => pad(v.render(get(o, k)), v.pad)).join(' | ');
};

const performQuery = (firestore) => (query) => {
  const { iterateCollection, iterateCollectionGroup, iterateSubCollection } = genIteratorFuncs(firestore);

  const col = query.from;
  const colGroup = !!query.collectionGroup;
  const colOpts = {
    where: query.where,
    orderBy: query.orderBy,
    pageSize: query.pageSize,
    select: query.select && query.select.map((s) => s.path || s),
  };
  const defCollectionHandler = (c) => log(c.id, tline(fsd(c), query.select));
  const handler = query.handler || defCollectionHandler;

  if (colGroup) return iterateCollectionGroup(col, handler, colOpts);
  if (!query.subcollection) return iterateCollection(col, handler, colOpts);

  const subcol = query.subcollection.from;
  const subcolOpts = {
    where: query.subcollection.where,
    orderBy: query.subcollection.orderBy,
    pageSize: query.subcollection.pageSize,
    select: query.subcollection.select && query.subcollection.select.map((s) => s.path || s),
  };
  const defSubCollectionHandler = (sc, c) => log(c.id, sc.id, tline(fsd(sc), query.subcollection.select));
  const subcolhandler = query.subcollection.handler || defSubCollectionHandler;

  return iterateSubCollection(col, subcol, subcolhandler, colOpts, subcolOpts);
};

module.exports = {
  performQuery,
  tline,
  fsd,
  log,
};
