const { createBoardLeftActivity, createBoardJoinedActivity } = require('../../models/boards/board-activities');
const CollectionPaths = require('../../firestore-paths/CollectionPaths');

const genBoardActivityFuncs = ({ firestore }) => {
  /**
   * Get board activities collection reference.
   *
   * @param {string} boardId
   * @returns {FirebaseFirestore.CollectionReference<FirebaseFirestore.DocumentData>}
   */
  const getActivitiesCollectionRef = (boardId) => firestore.collection(CollectionPaths.BoardActivities({ boardId }));

  /**
   * Add board activity to sub collection "activities" of the board.
   *
   * @param {object} opts
   * @param {string} opts.boardId
   * @param {*} opts.activity
   *
   * @returns {Promise<FirebaseFirestore.DocumentReference<FirebaseFirestore.DocumentData>>}
   */
  const addBoardActivity = async ({ boardId, activity }) => {
    const boardsActivitiesRef = getActivitiesCollectionRef(boardId);
    return boardsActivitiesRef.add(activity);
  };

  /**
   * Add board left activity.
   *
   * @param {object} opts
   * @param {FirebaseFirestore.DocumentSnapshot} opts.boardSnap
   * @param {string} opts.createdBy
   * @param {string} opts.participantUid
   * @param {string} opts.participantName
   *
   * @returns {Promise<FirebaseFirestore.DocumentReference<FirebaseFirestore.DocumentData>>}
   */
  const addBoardLeftActivity = async ({ boardSnap, createdBy, participantUid, participantName }) => {
    const activity = createBoardLeftActivity({
      boardSnap,
      createdBy,
      participantUid,
      participantName,
    });

    return addBoardActivity({ boardId: boardSnap.id, activity });
  };

  /**
   * Add board joined activity.
   *
   * @param {object} opts
   * @param {FirebaseFirestore.DocumentSnapshot} opts.boardSnap
   * @param {object|string} opts.invitee
   * @param {object} opts.inviter
   *
   * @returns {Promise<FirebaseFirestore.DocumentReference<FirebaseFirestore.DocumentData>>}
   */
  const addBordJoinedActivity = async ({ boardSnap, invitee, inviter }) => {
    const activity = createBoardJoinedActivity({ invitee, inviter });

    return addBoardActivity({ boardId: boardSnap.id, activity });
  };

  return {
    getActivitiesCollectionRef,
    addBoardActivity,
    addBoardLeftActivity,
    addBordJoinedActivity,
  };
};

module.exports = { genBoardActivityFuncs };
