import { Board, BoardGroup } from '../../schemas/firestore/objects/board';

type DeleteBoardParams = {
  boardId: string;
  uid: string;
};

type CheckUserHasRoleParams = {
  uid: string;
  roleName: string;
  boardSnap: FirebaseFirestore.DocumentSnapshot;
};

type ArchiveBoardParams = DeleteBoardParams;

type FetchBoardAndValidateParams = {
  boardId: string;
  uid: string;
  roleName: string;
};

type ChangeParticipantRole = {
  participantUid: string;
  roleValue: number;
};

type UpdateParticipantRoleParams = {
  boardSnap: FirebaseFirestore.DocumentSnapshot;
  roleNameOrValue: string;
  currentUserId: string;
  currentUserDisplayName: string;
  participantUid: string;
};

declare const ParticipantRoles: number[];
declare const ActiveParticipantRoles: number[];

type AddedBoard = {
  id: string;
  _slug: string;
};

type IsBoardViewerDto = {
  board: Pick<Board, 'title', 'participants', 'public'>;
  uid: string;
};

type IsBoardEditorDto = IsBoardViewerDto;
type IsBoardOwnerDto = IsBoardViewerDto;
type CanViewDto = IsBoardViewerDto;

type UpdateGroupsDto = {
  boardData: Pick<Board, 'groups'>;
  newItemId: string;
  groupSlug?: string;
};

export type GenBoardsFuncsResult = (opts: { firestore: FirebaseFirestore.Firestore }) => {
  isBoardViewer: ({ board, uid }: IsBoardViewerDto) => boolean;
  isBoardEditor: ({ board, uid }: IsBoardEditorDto) => boolean;
  isBoardOwner: ({ board, uid }: IsBoardOwnerDto) => boolean;
  canView: ({ board, uid }: CanViewDto) => boolean;
  checkUserHasRole({ uid, roleName, boardSnap }: CheckUserHasRoleParams): void;
  getBoardRef: (boardId: string) => FirebaseFirestore.DocumentReference;
  addBoard: (board: Record<string, unknown>) => Promise<AddedBoard>;
  fetchBoard: (
    boardIdOrRef: string | FirebaseFirestore.DocumentReference
  ) => Promise<FirebaseFirestore.DocumentSnapshot>;
  fetchBoardOrFail: (
    boardIdOrRef: string | FirebaseFirestore.DocumentReference
  ) => Promise<FirebaseFirestore.DocumentSnapshot>;
  fetchBoardAndValidate: ({
    boardId,
    uid,
    roleName,
  }: FetchBoardAndValidateParams) => Promise<FirebaseFirestore.DocumentSnapshot>;
  deleteBoard: ({ boardId, uid }: DeleteBoardParams) => Promise<void>;
  archiveBoard: ({ boardId, uid }: ArchiveBoardParams) => Promise<void>;
  deleteUserBoards: (userId: string, dryrun?: boolean) => Promise<unknown[]>;
  updateGroups: ({ boardData, newItemId, groupSlug }: UpdateGroupsDto) => BoardGroup[];
  changeParticipantRole: ({ participantUid, roleValue }: ChangeParticipantRole) => Record<string, unknown>;
  updateParticipantRole: ({
    boardSnap,
    roleNameOrValue,
    currentUserId,
    currentUserDisplayName,
    participantUid,
  }: UpdateParticipantRoleParams) => Promise<void>;
  getUserParticipatingBoards: (
    userId: string
  ) => Promise<FirebaseFirestore.QuerySnapshot<FirebaseFirestore.DocumentData>>;
  isHighlighted: ({ boardSnap }: { boardSnap: FirebaseFirestore.DocumentSnapshot }) => Promise<boolean>;
};

declare const genBoardFuncs: GenBoardsFuncsResult;
