import { HatsaLogger } from '@hatsa/log';
import { auth } from 'firebase-admin'; // eslint-disable-line import/no-extraneous-dependencies
import { Invite } from '../../schemas/firestore/objects/invites/invite';

type GenInvitesFuncsOpts = {
  firestore: FirebaseFirestore.Firestore;
  logger?: HatsaLogger;
};

type AcceptInviteOpts = {
  inviteId: string;
  invitee: auth.UserRecord;
  inviterId?: string;
};

type RejectInviteOpts = AcceptInviteOpts;
type FetchPendingInvitesOpts = {
  inviterId: string;
  email: string;
};

type AcceptInviteObj = Invite;
type RejectInviteObj = Invite;

type AcceptInviteResult = Promise<Array<null | AcceptInviteObj> | AcceptInviteObj>;
type RejectInviteResult = Promise<Array<null | RejectInviteObj> | RejectInviteObj>;

export type GenInvitesFuncsResult = (opts: GenInvitesFuncsOpts) => {
  fetchPendingInvites: ({ inviterId, email }: FetchPendingInvitesOpts) => Promise<FirebaseFirestore.QuerySnapshot>;
  fetchInvitesByIdOrFail: ({ inviteId }: { inviteId: string }) => Promise<FirebaseFirestore.QuerySnapshot>;
  accept: ({ inviteId, invitee, inviterId }: AcceptInviteOpts) => AcceptInviteResult;
  reject: ({ inviteId, invitee, inviterId }: RejectInviteOpts) => RejectInviteResult;
};

declare const genInvitesFuncs: GenInvitesFuncsResult;
