/* eslint-disable camelcase */

import type { HatsaLogger } from '@hatsa/log';

type RoleChangedNotificationDto = {
  targetUserId: string;
  initiatorId: string;
  initiatorName: string;
  boardSnap: FirebaseFirestore.DocumentSnapshot;
  roleValue: number;
};

type BoardReminderNotificationDto = {
  uid: string;
  board: {
    created_by: string;
    updated_at: string;
    remind_at: string;
    default_board: string;
    folder: string;
    poster: { assets: Record<string, unknown> };
    id: string;
  };
};

type EventOwnerNotificationDto = {
  ownerId: string;
  event: {
    id: string;
    title: string;
    created_by: string;
  };
};

type EventParticipantNotificationDto = Omit<EventOwnerNotificationDto, 'ownerId'> & {
  participantId: string;
};

type AddEventWithBoardNotificationDto = EventParticipantNotificationDto & {
  board: {
    id: string;
  };
};

type AddUserStartFollowingDto = {
  followingUid: string;
  followerUserSnap: FirebaseFirestore.DocumentSnapshot;
};

type FunctionOptions = {
  batch?: FirebaseFirestore.WriteBatch;
};

type AddEventOwnerNotification = (
  { ownerId, event }: EventOwnerNotificationDto,
  opts?: FunctionOptions
) => Promise<FirebaseFirestore.WriteResult> | Promise<FirebaseFirestore.WriteBatch>;

type AddEventWithBoardNotification = (
  { participantId, event, board }: AddEventWithBoardNotificationDto,
  opts?: FunctionOptions
) => Promise<FirebaseFirestore.WriteResult> | Promise<FirebaseFirestore.WriteBatch>;

export interface NotificationFuncs {
  addBoardParticipantRoleChangedNotification: ({
    targetUserId,
    initiatorId,
    initiatorName,
    boardSnap,
    roleValue,
  }: RoleChangedNotificationDto) => Promise<FirebaseFirestore.WriteResult>;
  addBoardReminder: (
    { uid, boardSnap }: BoardReminderNotificationDto,
    { batch }: FunctionOptions
  ) => Promise<FirebaseFirestore.WriteResult> | Promise<FirebaseFirestore.WriteBatch>;
  addEventNotApplied: AddEventOwnerNotification;
  addEventNotDrawn: AddEventOwnerNotification;
  addEventIsOver: AddEventWithBoardNotification;
  addEventBoardEmpty: AddEventWithBoardNotification;
  addUserStartFollowing: ({
    followingUid,
    followerUserSnap,
  }: AddUserStartFollowingDto) => Promise<FirebaseFirestore.WriteResult>;
}

declare const genNotificationFuncs: (opts: {
  firestore: FirebaseFirestore.Firestore;
  logger?: HatsaLogger;
  dryRun?: boolean;
}) => NotificationFuncs;
