const genOffererFuncs = () => {
  const isVerified = ({ offererSnap }) => !!offererSnap.get('verification.verified');

  const isManagingBy = ({ offererSnap, userId }) => Object.keys(offererSnap.get('managed_by') || {}).includes(userId);

  const hasActiveSubscription = ({ user }) => {
    const { subscriptions } = user.customClaims || {};

    if (!subscriptions || typeof subscriptions !== 'object') {
      return false;
    }

    return Object.values(subscriptions).some((endTimestamp) => new Date(endTimestamp * 1000) >= new Date());
  };

  const canManageShop = ({ offererSnap, user }) =>
    hasActiveSubscription({ user }) &&
    isVerified({ offererSnap }) &&
    isManagingBy({
      offererSnap,
      userId: user.uid,
    });

  return {
    isVerified,
    isManagingBy,
    hasActiveSubscription,
    canManageShop,
  };
};

module.exports = { genOffererFuncs };
