// const { tryRetry } = require('@hatsa/utils/promise');
const { findDocType } = require('./match');

module.exports = ({ esclient, logger }) => {
  const catchHandler = (err) => {
    if (logger) {
      logger.warn(`es error: ${err.message}`);
    }
  };

  const deindex = (docType, docId) => {
    if (logger) {
      logger.info(`deindexing ${docType} ${docId}`);
    }
    return esclient.deleteDoc(docType, docId).catch(catchHandler);
  };

  const index = (docType, doc) => {
    if (logger) {
      logger.info(`indexing ${docType} ${doc.id}`);
    }
    return esclient.indexDoc(docType, doc);
  };

  const updateIndexOnUpdate = (gDocType, gChange) => {
    let docType;
    let change;

    if (typeof gDocType === 'string') {
      docType = gDocType;
      change = gChange;
    } else {
      change = gDocType;
      docType = findDocType(change.before.ref.path);
    }

    if (!docType) throw new Error('could not determine docType');

    if (!change) throw new Error('no change object given');

    const newDoc = change.after.data();

    const docId = change.after.id;
    const isDelete = !change.after.exists || !!newDoc.deleted_at;

    if (isDelete) return deindex(docType, docId);

    return index(docType, change.after);
  };

  // const index = tryRetry(
  //     (docType, docId, docData) => {
  //         logger.info(`indexing ${docType} ${docId}`, docData);
  //         console.log(`indexing ${docType} ${docId} `);
  //         return esclient.indexDoc(docType, docId, docData)
  //         .then(ret => console.log('index complete: ', ret))
  //         ;
  //     },
  //     err => {
  //         console.warn(`ES indexing failed: (${err.message}), retrying...`);
  //         logger.warn(`ES indexing failed: (${err.message}), retrying...`);
  //         return true; // TODO: only retry on certain errors?
  //     }
  // );

  return {
    updateIndexOnUpdate,
    deindex,
    index,
  };
};
