const test = require('tape');
const sinon = require('sinon');

const { createChange } = require('./test')();
const genFsEsHelpers = require('./es');

test('updateIndexOnUpdate string/change', (t) => {
  const deleteDocSpy = sinon.spy();
  const indexDocSpy = sinon.spy();

  const { updateIndexOnUpdate } = genFsEsHelpers({
    esclient: {
      deleteDoc: deleteDocSpy,
      indexDoc: indexDocSpy,
    },
  });

  const change = createChange({ foo: 'bar' }, { foo: 'bax' }, 'foo/bar');

  updateIndexOnUpdate('boe', change);

  t.ok(indexDocSpy.calledWith('boe', change.after));

  t.end();
});

test('updateIndexOnUpdate change', (t) => {
  const deleteDocSpy = sinon.spy();
  const indexDocSpy = sinon.spy();

  const { updateIndexOnUpdate } = genFsEsHelpers({
    esclient: {
      deleteDoc: deleteDocSpy,
      indexDoc: indexDocSpy,
    },
  });

  const change = createChange({ foo: 'bar' }, { foo: 'bax' }, 'profiles/abc');

  updateIndexOnUpdate(change);

  t.ok(indexDocSpy.calledWith('profiles', change.after));

  t.end();
});

test('updateIndexOnUpdate change without docType', (t) => {
  const deleteDocSpy = sinon.spy();
  const indexDocSpy = sinon.spy();

  const { updateIndexOnUpdate } = genFsEsHelpers({
    esclient: {
      deleteDoc: deleteDocSpy,
      indexDoc: indexDocSpy,
    },
  });

  const change = createChange({ foo: 'bar' }, { foo: 'bax' }, 'hallo/abc');

  t.throws(() => updateIndexOnUpdate(change));

  t.end();
});
