const UrlPattern = require('url-pattern');
const { DocumentTypes, getCollectionPathByParams } = require('../match');

const CollectionPaths = Object.freeze({
  Assets: () => getCollectionPathByParams(DocumentTypes.Assets),
  AssetRefs: ({ assetId }) => getCollectionPathByParams(DocumentTypes.AssetRefs, { assetId }),
  Boards: () => getCollectionPathByParams(DocumentTypes.Boards),
  BoardItems: ({ boardId }) => getCollectionPathByParams(DocumentTypes.BoardItems, { boardid: boardId }),
  BoardActivities: ({ boardId }) => getCollectionPathByParams(DocumentTypes.BoardActivities, { boardid: boardId }),
  Invites: () => getCollectionPathByParams(DocumentTypes.Invites),
  // it's not possible to stringify UrlPattern which contains RegExp,
  // that's why we're using UrlPattern directly with slightly modified pattern
  InviteActions: ({ inviteId }) => new UrlPattern('invites/:inviteId/actions').stringify({ inviteId }),
  Offerers: () => getCollectionPathByParams(DocumentTypes.Offerers),
  Outclicks: ({ offererId }) => getCollectionPathByParams(DocumentTypes.Outclicks, { offerer: offererId }),
  Profiles: () => getCollectionPathByParams(DocumentTypes.Profiles),
  ProfileActions: ({ uid }) => getCollectionPathByParams(DocumentTypes.ProfileActions, { uid }),
  ProfileFollowing: ({ uid }) => getCollectionPathByParams(DocumentTypes.ProfileFollowing, { uid }),
  ProfileFollowers: ({ uid }) => getCollectionPathByParams(DocumentTypes.ProfileFollowers, { uid }),
  ProfileLikes: ({ uid }) => getCollectionPathByParams(DocumentTypes.ProfileLikes, { uid }),
  ProfileFulfilled: ({ uid }) => getCollectionPathByParams(DocumentTypes.ProfileFulfilled, { uid }),
  ProfileDevices: ({ uid }) => getCollectionPathByParams(DocumentTypes.ProfileDevices, { uid }),
  ProfileNotifications: ({ uid }) => getCollectionPathByParams(DocumentTypes.ProfileNotifications, { uid }),
  ProfileInvites: ({ uid }) => getCollectionPathByParams(DocumentTypes.ProfileInvites, { uid }),
  ProfileSlugs: () => getCollectionPathByParams(DocumentTypes.ProfileSlugs),
  UrlRefs: () => getCollectionPathByParams(DocumentTypes.UrlRefs),
  UrlRefsRefs: ({ sha1 }) => getCollectionPathByParams(DocumentTypes.UrlRefsRefs, { sha1 }),
  UrlRefMap: () => getCollectionPathByParams(DocumentTypes.UrlRefMap),
  UrlRefScrapes: ({ sha1 }) => getCollectionPathByParams(DocumentTypes.UrlRefScrapes, { sha1 }),
  Affiliates: () => getCollectionPathByParams(DocumentTypes.Affiliates),
  AffiliatePrograms: ({ affiliateId }) =>
    getCollectionPathByParams(DocumentTypes.AffiliatePrograms, { affiliate: affiliateId }),
  AffiliatePromotions: ({ affiliateId, programId }) =>
    getCollectionPathByParams(DocumentTypes.AffiliatePromotions, { affiliate: affiliateId, programId }),
  AffiliateClicks: ({ affiliateId, programId }) =>
    getCollectionPathByParams(DocumentTypes.AffiliateClicks, { affiliate: affiliateId, programId }),
  AffiliateFeeds: ({ affiliateId, programId }) =>
    getCollectionPathByParams(DocumentTypes.AffiliateFeeds, { affiliate: affiliateId, programId }),
  Products: ({ affiliateId, programId, feedId }) =>
    getCollectionPathByParams(DocumentTypes.Products, { affiliate: affiliateId, programId, feedId }),
});

module.exports = CollectionPaths;
