const test = require('tape');
const CollectionPaths = require('./CollectionPaths');
const { Mocks } = require('./firestore-path.test.data');

const testCases = {
  Assets: {
    expected: 'assets',
  },
  AssetRefs: {
    args: { assetId: Mocks.assetId },
    expected: `assets/${Mocks.assetId}/refs`,
  },
  Boards: {
    expected: 'boards',
  },
  BoardItems: {
    args: { boardId: Mocks.boardId },
    expected: `boards/${Mocks.boardId}/items`,
  },
  BoardActivities: {
    args: { boardId: Mocks.boardId },
    expected: `boards/${Mocks.boardId}/activities`,
  },
  Invites: {
    expected: 'invites',
  },
  InviteActions: {
    args: { inviteId: Mocks.inviteId },
    expected: `invites/${Mocks.inviteId}/actions`,
  },
  Offerers: {
    expected: 'offerers',
  },
  Outclicks: {
    args: { offererId: Mocks.offererId },
    expected: `offerers/${Mocks.offererId}/outclicks`,
  },
  Profiles: {
    expected: 'profiles',
  },
  ProfileActions: {
    args: { uid: Mocks.uid },
    expected: `profiles/${Mocks.uid}/actions`,
  },
  ProfileFollowing: {
    args: { uid: Mocks.uid },
    expected: `profiles/${Mocks.uid}/following`,
  },
  ProfileFollowers: {
    args: { uid: Mocks.uid },
    expected: `profiles/${Mocks.uid}/followers`,
  },
  ProfileLikes: {
    args: { uid: Mocks.uid },
    expected: `profiles/${Mocks.uid}/likes`,
  },
  ProfileFulfilled: {
    args: { uid: Mocks.uid },
    expected: `profiles/${Mocks.uid}/fulfilled`,
  },
  ProfileDevices: {
    args: { uid: Mocks.uid },
    expected: `profiles/${Mocks.uid}/devices`,
  },
  ProfileNotifications: {
    args: { uid: Mocks.uid },
    expected: `profiles/${Mocks.uid}/notifications`,
  },
  ProfileInvites: {
    args: { uid: Mocks.uid },
    expected: `profiles/${Mocks.uid}/invites`,
  },
  ProfileSlugs: {
    expected: 'profile-slugs',
  },
  UrlRefs: {
    expected: 'urlrefs',
  },
  UrlRefsRefs: {
    args: { sha1: Mocks.sha1 },
    expected: `urlrefs/${Mocks.sha1}/refs`,
  },
  UrlRefMap: {
    expected: 'urlrefmap',
  },
  UrlRefScrapes: {
    args: { sha1: Mocks.sha1 },
    expected: `urlrefs/${Mocks.sha1}/scrapes`,
  },
  Affiliates: {
    expected: 'affiliates',
  },
  AffiliatePrograms: {
    args: { affiliateId: Mocks.affiliateId },
    expected: `affiliates/${Mocks.affiliateId}/affiliate_programs`,
  },
  AffiliatePromotions: {
    args: { affiliateId: Mocks.affiliateId, programId: Mocks.programId },
    expected: `affiliates/${Mocks.affiliateId}/affiliate_programs/${Mocks.programId}/affiliate_promotions`,
  },
  AffiliateClicks: {
    args: { affiliateId: Mocks.affiliateId, programId: Mocks.programId },
    expected: `affiliates/${Mocks.affiliateId}/affiliate_programs/${Mocks.programId}/affiliate_clicks`,
  },
  AffiliateFeeds: {
    args: { affiliateId: Mocks.affiliateId, programId: Mocks.programId },
    expected: `affiliates/${Mocks.affiliateId}/affiliate_programs/${Mocks.programId}/affiliate_feeds`,
  },
  Products: {
    args: { affiliateId: Mocks.affiliateId, programId: Mocks.programId, feedId: Mocks.feedId },
    expected: `affiliates/${Mocks.affiliateId}/affiliate_programs/${Mocks.programId}/affiliate_feeds/${Mocks.feedId}/affiliate_products`,
  },
};

test('[CollectionPaths] should build a valid collections paths', (t) => {
  Object.entries(testCases).forEach(([funcName, { args, expected }]) => {
    t.equals(
      CollectionPaths[funcName](args),
      expected,
      `'CollectionPaths.${funcName}(${
        args ? `{ ${Object.keys(args).join(', ')} }` : ''
      })' should build path - '${expected}'`
    );
  });

  t.end();
});
