const UrlPattern = require('url-pattern');
const { DocumentTypes, getDocPathByParams } = require('../match');

const DocumentPaths = Object.freeze({
  Assets: ({ assetId }) => getDocPathByParams(DocumentTypes.Assets, { publicid: assetId }),
  AssetRefs: ({ assetId, refId }) => getDocPathByParams(DocumentTypes.AssetRefs, { assetId, refId }),
  Boards: ({ boardId }) => getDocPathByParams(DocumentTypes.Boards, { boardid: boardId }),
  BoardItems: ({ boardId, itemId }) =>
    getDocPathByParams(DocumentTypes.BoardItems, {
      boardid: boardId,
      itemid: itemId,
    }),
  BoardActivities: ({ boardId, activityId }) =>
    getDocPathByParams(DocumentTypes.BoardActivities, {
      boardid: boardId,
      activityid: activityId,
    }),
  // it's not possible to stringify UrlPattern which contains RegExp,
  // that's why we're using UrlPattern directly with slightly modified pattern
  Invites: ({ inviteId }) => new UrlPattern('invites/:inviteId').stringify({ inviteId }),
  InviteActions: ({ inviteId, actionId }) =>
    new UrlPattern('invites/:inviteId/actions/:actionId').stringify({
      inviteId,
      actionId,
    }),
  Offerers: ({ offererId }) => new UrlPattern('offerers/:offererId').stringify({ offererId }),
  Outclicks: ({ offererId, outclickId }) =>
    getDocPathByParams(DocumentTypes.Outclicks, {
      offerer: offererId,
      clickid: outclickId,
    }),
  Profiles: ({ uid }) => getDocPathByParams(DocumentTypes.Profiles, { uid }),
  ProfileActions: ({ uid, actionId }) => getDocPathByParams(DocumentTypes.ProfileActions, { uid, actionId }),
  ProfileFollowing: ({ uid, followUid }) => getDocPathByParams(DocumentTypes.ProfileFollowing, { uid, followUid }),
  ProfileFollowers: ({ uid, followUid }) => getDocPathByParams(DocumentTypes.ProfileFollowers, { uid, followUid }),
  ProfileLikes: ({ uid, likeId }) => getDocPathByParams(DocumentTypes.ProfileLikes, { uid, pathId: likeId }),
  ProfileFulfilled: ({ uid, itemId }) => getDocPathByParams(DocumentTypes.ProfileFulfilled, { uid, itemId }),
  ProfileDevices: ({ uid, hardwareIdSha1 }) =>
    getDocPathByParams(DocumentTypes.ProfileDevices, {
      uid,
      hardwareIdSha1,
    }),
  ProfileNotifications: ({ uid, notificationId }) =>
    getDocPathByParams(DocumentTypes.ProfileNotifications, {
      uid,
      notificationId,
    }),
  ProfileInvites: ({ uid, inviteId }) =>
    getDocPathByParams(DocumentTypes.ProfileInvites, {
      uid,
      inviteId,
    }),
  ProfileSlugs: ({ slugId }) => new UrlPattern('profile-slugs/:slugId').stringify({ slugId }),
  UrlRefs: ({ sha1 }) => getDocPathByParams(DocumentTypes.UrlRefs, { sha1 }),
  UrlRefsRefs: ({ sha1, refId }) => getDocPathByParams(DocumentTypes.UrlRefsRefs, { sha1, refId }),
  UrlRefMap: ({ sha1 }) => getDocPathByParams(DocumentTypes.UrlRefMap, { sha1 }),
  UrlRefScrapes: ({ sha1, scrapeId }) => getDocPathByParams(DocumentTypes.UrlRefScrapes, { sha1, scrapeId }),
  Affiliates: ({ affiliateId }) => getDocPathByParams(DocumentTypes.Affiliates, { affiliate: affiliateId }),
  AffiliatePrograms: ({ affiliateId, programId }) =>
    getDocPathByParams(DocumentTypes.AffiliatePrograms, {
      affiliate: affiliateId,
      programId,
    }),
  AffiliatePromotions: ({ affiliateId, programId, promotionId }) =>
    getDocPathByParams(DocumentTypes.AffiliatePromotions, { affiliate: affiliateId, programId, promotionId }),
  AffiliateClicks: ({ affiliateId, programId, clickId }) =>
    getDocPathByParams(DocumentTypes.AffiliateClicks, { affiliate: affiliateId, programId, clickId }),
  AffiliateFeeds: ({ affiliateId, programId, feedId }) =>
    getDocPathByParams(DocumentTypes.AffiliateFeeds, { affiliate: affiliateId, programId, feedId }),
  Products: ({ affiliateId, programId, feedId, productId }) =>
    getDocPathByParams(DocumentTypes.Products, { affiliate: affiliateId, programId, feedId, productId }),
});

module.exports = DocumentPaths;
