const test = require('tape');
const DocumentPaths = require('./DocumentPaths');
const { Mocks } = require('./firestore-path.test.data');

const testCases = {
  Assets: {
    args: { assetId: Mocks.assetId },
    expected: `assets/${Mocks.assetId}`,
  },
  AssetRefs: {
    args: { assetId: Mocks.assetId, refId: Mocks.refId },
    expected: `assets/${Mocks.assetId}/refs/${Mocks.refId}`,
  },
  Boards: {
    args: { boardId: Mocks.boardId },
    expected: `boards/${Mocks.boardId}`,
  },
  BoardItems: {
    args: { boardId: Mocks.boardId, itemId: Mocks.itemId },
    expected: `boards/${Mocks.boardId}/items/${Mocks.itemId}`,
  },
  BoardActivities: {
    args: { boardId: Mocks.boardId, activityId: Mocks.activityId },
    expected: `boards/${Mocks.boardId}/activities/${Mocks.activityId}`,
  },
  Invites: {
    args: { inviteId: Mocks.inviteId },
    expected: `invites/${Mocks.inviteId}`,
  },
  InviteActions: {
    args: { inviteId: Mocks.inviteId, actionId: Mocks.actionId },
    expected: `invites/${Mocks.inviteId}/actions/${Mocks.actionId}`,
  },
  Offerers: {
    args: { offererId: Mocks.offererId },
    expected: `offerers/${Mocks.offererId}`,
  },
  Outclicks: {
    args: { offererId: Mocks.offererId, outclickId: Mocks.outclickId },
    expected: `offerers/${Mocks.offererId}/outclicks/${Mocks.outclickId}`,
  },
  Profiles: {
    args: { uid: Mocks.uid },
    expected: `profiles/${Mocks.uid}`,
  },
  ProfileActions: {
    args: { uid: Mocks.uid, actionId: Mocks.actionId },
    expected: `profiles/${Mocks.uid}/actions/${Mocks.actionId}`,
  },
  ProfileFollowing: {
    args: { uid: Mocks.uid, followUid: Mocks.followUid },
    expected: `profiles/${Mocks.uid}/following/${Mocks.followUid}`,
  },
  ProfileFollowers: {
    args: { uid: Mocks.uid, followUid: Mocks.followUid },
    expected: `profiles/${Mocks.uid}/followers/${Mocks.followUid}`,
  },
  ProfileLikes: {
    args: { uid: Mocks.uid, likeId: Mocks.likeId },
    expected: `profiles/${Mocks.uid}/likes/${Mocks.likeId}`,
  },
  ProfileFulfilled: {
    args: { uid: Mocks.uid, itemId: Mocks.itemId },
    expected: `profiles/${Mocks.uid}/fulfilled/${Mocks.itemId}`,
  },
  ProfileDevices: {
    args: { uid: Mocks.uid, hardwareIdSha1: Mocks.hardwareIdSha1 },
    expected: `profiles/${Mocks.uid}/devices/${Mocks.hardwareIdSha1}`,
  },
  ProfileNotifications: {
    args: { uid: Mocks.uid, notificationId: Mocks.notificationId },
    expected: `profiles/${Mocks.uid}/notifications/${Mocks.notificationId}`,
  },
  ProfileInvites: {
    args: { uid: Mocks.uid, inviteId: Mocks.inviteId },
    expected: `profiles/${Mocks.uid}/invites/${Mocks.inviteId}`,
  },
  ProfileSlugs: {
    args: { slugId: Mocks.slugId },
    expected: `profile-slugs/${Mocks.slugId}`,
  },
  UrlRefs: {
    args: { sha1: Mocks.sha1 },
    expected: `urlrefs/${Mocks.sha1}`,
  },
  UrlRefsRefs: {
    args: { sha1: Mocks.sha1, refId: Mocks.refId },
    expected: `urlrefs/${Mocks.sha1}/refs/${Mocks.refId}`,
  },
  UrlRefMap: {
    args: { sha1: Mocks.sha1 },
    expected: `urlrefmap/${Mocks.sha1}`,
  },
  UrlRefScrapes: {
    args: { sha1: Mocks.sha1, scrapeId: Mocks.scrapeId },
    expected: `urlrefs/${Mocks.sha1}/scrapes/${Mocks.scrapeId}`,
  },
  Affiliates: {
    args: { affiliateId: Mocks.affiliateId },
    expected: `affiliates/${Mocks.affiliateId}`,
  },
  AffiliatePrograms: {
    args: { affiliateId: Mocks.affiliateId, programId: Mocks.programId },
    expected: `affiliates/${Mocks.affiliateId}/affiliate_programs/${Mocks.programId}`,
  },
  AffiliatePromotions: {
    args: { affiliateId: Mocks.affiliateId, programId: Mocks.programId, promotionId: Mocks.promotionId },
    expected: `affiliates/${Mocks.affiliateId}/affiliate_programs/${Mocks.programId}/affiliate_promotions/${Mocks.promotionId}`,
  },
  AffiliateClicks: {
    args: { affiliateId: Mocks.affiliateId, programId: Mocks.programId, clickId: Mocks.clickId },
    expected: `affiliates/${Mocks.affiliateId}/affiliate_programs/${Mocks.programId}/affiliate_clicks/${Mocks.clickId}`,
  },
  AffiliateFeeds: {
    args: { affiliateId: Mocks.affiliateId, programId: Mocks.programId, feedId: Mocks.feedId },
    expected: `affiliates/${Mocks.affiliateId}/affiliate_programs/${Mocks.programId}/affiliate_feeds/${Mocks.feedId}`,
  },
  Products: {
    args: {
      affiliateId: Mocks.affiliateId,
      programId: Mocks.programId,
      feedId: Mocks.feedId,
      productId: Mocks.productId,
    },
    expected: `affiliates/${Mocks.affiliateId}/affiliate_programs/${Mocks.programId}/affiliate_feeds/${Mocks.feedId}/affiliate_products/${Mocks.productId}`,
  },
};

test('[DocumentPaths] should build a valid documents paths', (t) => {
  Object.entries(testCases).forEach(([funcName, { args, expected }]) => {
    t.equals(
      DocumentPaths[funcName](args),
      expected,
      `'DocumentPaths.${funcName}(${
        args ? `{ ${Object.keys(args).join(', ')} }` : ''
      })' should build path - '${expected}'`
    );
  });

  t.end();
});
