const partialRight = require('lodash/partialRight');
const cloneWith = require('lodash/cloneDeepWith');
const isString = require('lodash/isString');

const { collect, replace } = require('@hatsa/utils/object');

const stringToParts = (s) =>
  isString(s)
    ? s
        .trim()
        .replace(/(^\/|\/$)/, '')
        .split('/')
    : [];
const stringIsDoc = (s) => stringToParts(s).length % 2 === 0;
const stringIsCollection = (s) => stringToParts(s).length % 2 === 1;
const stringCollectionParts = (s) => stringToParts(s).filter((p, i) => i % 2 === 0);

// Accepts a Doc or a DocRef and return it's path as a string
const refToPath = (ref) => {
  if (!ref) return null;

  if (typeof ref.path === 'string') return ref.path;

  if (ref.ref && ref.ref.path) return ref.ref.path;

  return null;
};

const toRef = (d, firestore) => (typeof d === 'string' ? firestore.doc(d) : d);
const toPath = (d) => (typeof d === 'string' ? d : refToPath(d));

const collectRefs = partialRight(collect, refToPath);

const toDate = (d) => (d && typeof d.toDate === 'function' ? d.toDate() : undefined);

const toJsDate = (data) =>
  replace(
    data,
    (d) => d && typeof d.toDate === 'function',
    (d) => d.toDate()
  );

const fsReplacer = (firestore) => (data) => {
  if (data instanceof firestore.constructor.Timestamp) return data.toDate();

  if (data instanceof firestore.constructor.DocumentReference) return data.path;

  // TODO: fix Blob types
  // if (data instanceof firestore.constructor.Blob)
  //     return data.toUint8Array();

  return undefined;
};

const snapToPojo = (snap) => cloneWith(snap.data(), fsReplacer(snap.ref.firestore));
const dataToPojo = (data, fs) => cloneWith(data, fsReplacer(fs));

module.exports = {
  refToPath,
  toRef,
  toPath,
  collectRefs,
  toDate,
  toJsDate,
  snapToPojo,
  dataToPojo,

  stringIsDoc,
  stringIsCollection,
  stringCollectionParts,
};
