const test = require('tape');
const { firestore } = require('firebase-functions-test')();

const {
  refToPath,
  toRef,
  toPath,
  // collectRefs,
  toJsDate,
  snapToPojo,
  dataToPojo,

  stringIsDoc,
  stringIsCollection,
  stringCollectionParts,
} = require('./index');

test('refToPath', (t) => {
  const docSnap1 = firestore.makeDocumentSnapshot({ foo: 'bar' }, 'foo/bar');
  const docSnap2 = firestore.makeDocumentSnapshot({ foo: 'bar' }, 'foo/bar/baz');

  t.equal(refToPath(docSnap1.ref), 'foo/bar');
  t.equal(refToPath(docSnap2.ref), 'foo/bar/baz');

  t.end();
});

test('toRef', (t) => {
  const docSnap1 = firestore.makeDocumentSnapshot({}, 'foo/bar');
  t.equal(toRef('foo/bar', docSnap1.ref.firestore).path, docSnap1.ref.path);
  t.equal(toRef(docSnap1.ref), docSnap1.ref);

  t.end();
});

test('toPath', (t) => {
  const docSnap1 = firestore.makeDocumentSnapshot({}, 'foo/bar');
  t.equal(toPath(docSnap1.ref), 'foo/bar');
  t.equal(toPath('foo/bar'), 'foo/bar');

  t.end();
});

test('toJsDate', (t) => {
  const now = new Date();

  const docSnap1 = firestore.makeDocumentSnapshot({ date: now }, 'foo/bar');
  t.deepEqual(toJsDate(docSnap1.data()), { date: now });

  t.end();
});

test('snapToPojo', (t) => {
  const now = new Date();
  const docSnap1 = firestore.makeDocumentSnapshot({ date: now }, 'foo/bar');
  t.deepEqual(snapToPojo(docSnap1), { date: now });

  //TODO: DocumentReference is not yet supported in firestore test framework
  // const docRef1 = firestore.exampleDocumentSnapshot().ref;
  // const docSnap2 = firestore.makeDocumentSnapshot({ docRef1 }, 'foo/bar');
  // t.deepEqual(snapToPojo(docSnap2), {docRef1: 'firestoreReference:records/1234'});

  //TODO: fix Blob types
  // const data_buf = new Buffer('data_buf');
  // const data_bar = new Uint8Array('data_bar');
  // const docSnap2 = firestore.makeDocumentSnapshot({  data_buf, data_bar }, 'foo/bar');
  // t.deepEqual(snapToPojo(docSnap2), { data_buf, data_bar });

  t.end();
});

test('dataToPojo', (t) => {
  const now = new Date();
  const docSnap1 = firestore.makeDocumentSnapshot({ date: now }, 'foo/bar');
  const data = docSnap1.data();
  const fs = docSnap1.ref.firestore;

  t.deepEqual(dataToPojo(data, fs), { date: now });

  t.end();
});

test('stringIsDoc', (t) => {
  t.ok(stringIsDoc('foo/1'));
  t.ok(stringIsDoc('foo/1/bar/2'));
  t.ok(stringIsDoc('  foo/1/bar/2')); // leading white space
  t.ok(stringIsDoc('foo/1/bar/2  ')); // trailing white space
  t.ok(stringIsDoc('/foo/1/bar/2')); // leading slash
  t.ok(stringIsDoc('foo/1/bar/2/')); // trailing slash

  t.notOk(stringIsDoc('foo'));
  t.notOk(stringIsDoc('foo/bar/1'));
  t.notOk(stringIsDoc('foo/1/bar'));

  t.end();
});

test('stringIsCollection', (t) => {
  t.ok(stringIsCollection('foo'));
  t.ok(stringIsCollection('foo/1/bar'));

  t.notOk(stringIsCollection('foo/1'));
  t.notOk(stringIsCollection('foo/1/bar/1'));

  t.end();
});

test('stringCollectionParts', (t) => {
  t.deepEqual(stringCollectionParts('foo'), ['foo']);
  t.deepEqual(stringCollectionParts('foo/1'), ['foo']);
  t.deepEqual(stringCollectionParts('foo/1/bar'), ['foo', 'bar']);
  t.deepEqual(stringCollectionParts('foo/1/bar/2'), ['foo', 'bar']);

  t.end();
});
