const isEqual = require('lodash/isEqual');
const omit = require('lodash/omit');
const pick = require('lodash/pick');

const diff = require('jsondiffpatch');

const { snapToPojo } = require('.');

const isDifferentOmit = (change, omits) => {
  const newData = change.after.data();
  const prevData = change.before.data();

  if (isEqual(omit(prevData, omits), omit(newData, omits))) return false;

  return true;
};

const isDifferentPick = (change, picks) => {
  const newData = change.after.data();
  const prevData = change.before.data();

  if (isEqual(pick(prevData, picks), pick(newData, picks))) return false;

  return true;
};

const getDiff = (change) => {
  // const newData  = change.after.data();
  // const prevData = change.before.data();
  const newData = snapToPojo(change.after);
  const prevData = snapToPojo(change.before);

  const delta = diff.diff(prevData, newData);
  return delta;
};

const isDifferentOnly = (change, key) => {
  if (!key || !change) return false;

  const diffKeys = Object.keys(getDiff(change) || {});

  if (diffKeys.length !== 1) return false;

  if (diffKeys[0] === key) return true;

  return false;
};

const genChangeDeltaString = (change) => JSON.stringify({ doc: change.after.ref.path, delta: getDiff(change) });

const logChangeDelta = (change, logger) => {
  const delta = getDiff(change);

  if (logger) logger.info(`change_delta: ${change.after.ref.path}`, getDiff(change));
  else diff.console.log(delta);
};

module.exports = {
  isDifferentOmit,
  isDifferentPick,
  isDifferentOnly,

  getDiff,
  logChangeDelta,

  genChangeDeltaString,
};
