const test = require('tape');
const { firestore, makeChange } = require('firebase-functions-test')();
const {
  getDiff,
  isDifferentOnly,
  // logDiff
} = require('./isDifferent');

test('getDiff date', (t) => {
  const date1 = new Date();
  const date2 = new Date(273643232342);

  const docSnap1 = firestore.makeDocumentSnapshot({ date: date1 }, 'foo/bar');
  const docSnap2 = firestore.makeDocumentSnapshot({ date: date2 }, 'foo/baz');

  t.deepEqual(getDiff(makeChange(docSnap1, docSnap2)), { date: [date1, date2] });
  // logDiff(makeChange(docSnap1, docSnap2));

  t.end();
});

const createSnap = (d) => firestore.makeDocumentSnapshot(d, 'foo/bar');
const createChange = (a, b) => makeChange(createSnap(a), createSnap(b));

test('isDifferentOnly', (t) => {
  const date = new Date();

  t.ok(isDifferentOnly(createChange({ foo: 1, touched: date }, { foo: 1 }), 'touched'));
  t.ok(isDifferentOnly(createChange({ foo: 1 }, { foo: 1, touched: date }), 'touched'));
  t.ok(isDifferentOnly(createChange({ foo: 1, touched: date }, { foo: 1, touched: 2 }), 'touched'));
  t.notOk(isDifferentOnly(createChange({ foo: 2, touched: date }, { foo: 1 }), 'touched'));
  t.notOk(isDifferentOnly(createChange({ foo: 1 }, { foo: 1 }), 'touched'));
  t.notOk(isDifferentOnly(createChange({ foo: { bar: 2 } }, { foo: { bar: 3 } }), 'touched'));
  t.notOk(isDifferentOnly(createChange({}, {}), 'foo'));

  t.end();
});
