import type { HatsaLogger } from '@hatsa/log';
import type { firestore } from 'firebase-admin';

export type WhereFilterOp =
  | '<'
  | '<='
  | '=='
  | '!='
  | '>='
  | '>'
  | 'array-contains'
  | 'in'
  | 'not-in'
  | 'array-contains-any';

type QueryWhere = [string | firestore.FieldPath, WhereFilterOp, unknown];

type IterateCollectionOpts<T> = {
  orderBy?: string;
  orderDir?: string;
  pageSize?: number;
  concurrency?: number;
  select?: string[];
  where?: QueryWhere | QueryWhere[];
  query?: FirebaseFirestore.Query<T>;
  logger?: HatsaLogger;
};

interface GenIteratorFuncs {
  /**
   * Iterate over the (sub)collection in a promise-like or stream-like way.
   * @returns array of document ids
   */
  iterateCollection: <T>(
    collection: string,
    cb: (docSnap: FirebaseFirestore.DocumentSnapshot) => void,
    opts: IterateCollectionOpts<T>
  ) => Promise<string>;
}

// eslint-disable-next-line import/prefer-default-export
export const genIteratorFuncs: (firestore: FirebaseFirestore.Firestore) => GenIteratorFuncs;
