const UrlPattern = require('url-pattern');
const map = require('lodash/map');

const { stringIsDoc, stringIsCollection, stringCollectionParts } = require('../index'); // export for backwards compatibility

const urlPatternCollectionParts = (urlPattern) => {
  if (!urlPattern.col.isRegex) {
    return urlPattern.col.ast.filter((a) => a.tag === 'static').map((a) => a.value.replace('/', ''));
  }

  return urlPattern.col.regex.toString().match(/\w+/g);
};

const genMatcher = (gp) => {
  const patterns = map(gp, (v, k) => ({
    type: k,
    doc: v.doc,
    col: v.col,
    colgroup: v.colgroup,
  }));

  const findType = (i, t) => (patterns.find((pattern) => pattern[t].match(i)) || {}).type;
  const findDocType = (i) => findType(i, 'doc');
  const findCollectionType = (i) => findType(i, 'col');
  const findCollectionGroup = (i) => (patterns.find((pattern) => pattern.colgroup === i) || {}).type;

  const getParamsByType = (i) => {
    let ret;

    patterns.find((p) => {
      ['col', 'doc'].forEach((t) => {
        const m = p[t].match(i);

        if (m) {
          ret = {
            type: t,
            docType: p.type,
            params: m,
          };
        }
      });
      return ret && true;
    });

    return ret;
  };

  const getDocOrColPathByParams = (docOrCol) => (type, params) => {
    const fType = gp[type];

    if (!fType) throw new Error(`no such type: ${type}`);

    return fType[docOrCol].stringify(params);
  };

  const getDocPathByParams = getDocOrColPathByParams('doc');
  const getCollectionPathByParams = getDocOrColPathByParams('col');

  return {
    findDocType,
    findCollectionType,
    findCollectionGroup,
    getParamsByType,
    getDocPathByParams,
    getCollectionPathByParams,
  };
};

module.exports = {
  UrlPattern,
  genMatcher,
  urlPatternCollectionParts,

  // export for backwards compatibility
  stringIsDoc,
  stringIsCollection,
  stringCollectionParts,
};
