const get = require('lodash/get');
const isEqual = require('lodash/isEqual');
const { isDifferentOnly } = require('../isDifferent');

const isPropertyCreated = (change, propName) =>
  !!(typeof change.before.get(propName) === 'undefined' && typeof change.after.get(propName) !== 'undefined');
const isPropertyChanged = (change, propName) =>
  isPropertyCreated(change, propName) || !isEqual(change.before.get(propName), change.after.get(propName));
const isTouched = (change) => isPropertyCreated(change, 'touch');
const isUntouchedOnly = (change) => !isTouched(change) && isDifferentOnly(change, 'touch');
const isDeleted = (change) => !change.after.exists;
const isSoftDeleted = (change) =>
  (isPropertyCreated(change, 'deleted_at') && change.after.get('deleted_at') !== null) ||
  (change.before.get('deleted_at') === null && change.after.get('deleted_at') !== null);
const isCreated = (change) => !change.before.exists && change.after.exists;
const isUpdated = (change) => change.before.exists && change.after.exists;

const getDoc = (change) => change.after;
const getRef = (change) => change.after.ref;
const getFirestore = (change) => change.after.ref.firestore;

const getParams = (context) => get(context, 'params', {});
const getParam = (context, param) => getParams(context)[param];

const baseChange = (change, context) => ({
  isTouched: isTouched(change),
  isPropertyCreated: (propName) => isPropertyCreated(change, propName),
  isPropertyChanged: (propName) => isPropertyChanged(change, propName),
  isUntouchedOnly: isUntouchedOnly(change),
  isDeleted: isDeleted(change),
  isSoftDeleted: isSoftDeleted(change),
  isCreated: isCreated(change),
  isUpdated: isUpdated(change),
  getRef: getRef(change),
  getFirestore: getFirestore(change),
  getParams: getParams(context),
  getParam: getParam(context),
  getDoc: getDoc(change),
});

module.exports = baseChange;
module.exports.isTouched = isTouched;
module.exports.isPropertyCreated = isPropertyCreated;
module.exports.isPropertyChanged = isPropertyChanged;
module.exports.isUntouchedOnly = isUntouchedOnly;
module.exports.isDeleted = isDeleted;
module.exports.isSoftDeleted = isSoftDeleted;
module.exports.isCreated = isCreated;
module.exports.isUpdated = isUpdated;
module.exports.getRef = getRef;
module.exports.getFirestore = getFirestore;
module.exports.getParams = getParams;
module.exports.getParam = getParam;
