const test = require('tape');
const sinon = require('sinon').createSandbox();
const { isSoftDeleted, isPropertyCreated } = require('./baseChange');

test('[isPropertyCreated] When property exists after, but not before the change', (t) => {
  const change = {
    before: { get: () => undefined },
    after: { get: () => 'mock value' },
  };

  t.equals(isPropertyCreated(change), true, 'should return `true`');
  t.end();

  sinon.restore();
});

test('[isPropertyCreated] When property exists after and before the change', (t) => {
  const change = {
    before: { get: () => 'mock value' },
    after: { get: () => 'mock value' },
  };

  t.equals(isPropertyCreated(change), false, 'should return `false`');
  t.end();

  sinon.restore();
});

test('[isSoftDeleted] When property `deleted_at` exists after, but not before the change', (t) => {
  const change = {
    before: { get: () => undefined },
    after: { get: () => '10/14/20, 11:27 AM' },
  };

  t.equals(isSoftDeleted(change), true, 'should return `true`');
  t.end();

  sinon.restore();
});

test('[isSoftDeleted] When property `deleted_at` exists after and before the change', (t) => {
  const change = {
    before: { get: () => '10/14/20, 11:27 AM' },
    after: { get: () => '10/14/20, 11:27 AM' },
  };

  t.equals(isSoftDeleted(change), false, 'should return `false`');
  t.end();

  sinon.restore();
});
