/**
 * Create board activity object.
 *
 * @param {string} type
 * @param {string} createdBy
 * @param {*} data
 *
 * @returns {*}
 */
const { CREATED_AT_PROP, CREATED_BY_PROP, UPDATED_AT_PROP } = require('@hatsa/constants');
const pickBy = require('lodash/pickBy');
const pick = require('lodash/pick');
const { getDisplayName } = require('../profiles');

const createBoardActivity = ({ type, createdBy, data }) => {
  const activity = {
    [CREATED_AT_PROP]: new Date(),
    [UPDATED_AT_PROP]: new Date(),
    type,
  };

  if (createdBy) activity[CREATED_BY_PROP] = createdBy;

  return { ...activity, ...(data || {}) };
};

/**
 * Create board left activity.
 *
 * @param {Object} opts
 * @param {FirebaseFirestore.DocumentSnapshot} opts.boardSnap
 * @param {string} opts.createdBy
 * @param {string} opts.participantUid
 * @param {string} opts.participantName
 *
 * @returns {*}
 */
const createBoardLeftActivity = ({ boardSnap, createdBy, participantUid, participantName }) =>
  createBoardActivity({
    type: 'left',
    createdBy,
    data: {
      user: {
        uid: participantUid,
        displayName: participantName,
      },
      meta: {
        board: {
          id: boardSnap.id,
          title: boardSnap.get('title'),
        },
      },
    },
  });

const createBoardJoinedActivity = ({ invitee, inviter }) => {
  const inviteeUser = pickBy(pick(invitee, ['uid', 'displayName', 'photoURL']));

  if (!inviteeUser.displayName) {
    inviteeUser.displayName = getDisplayName(invitee);
  }

  return createBoardActivity({
    type: 'joined',
    createdBy: !invitee || typeof invitee === 'string' ? null : invitee.uid,
    data: {
      invite: { invitee: inviteeUser },
      user: pick(inviter, ['uid', 'displayName', 'photoURL']),
    },
  });
};

module.exports = {
  createBoardActivity,
  createBoardLeftActivity,
  createBoardJoinedActivity,
};
