const { CREATED_BY_PROP } = require('@hatsa/constants');
const { ROLE_REVOKED_VALUE } = require('@hatsa/constants/sharing');
const { assignUpdatedProps, assignDeletedProps } = require('../../common/utils');
const { FolderTypes } = require('../../common/boards/folder-types');

/**
 * Set deleted board properties: the folder and participants roles.
 *
 * @param {object} opts
 * @param {FirebaseFirestore.DocumentSnapshot} opts.boardSnap - The board snapshot.
 * @param {string} opts.uid - ID of the user who's updating the board.
 * @param {string} opts.folder - The folder property.
 */
const setBoardFolder = ({ boardSnap, uid, folder }) => {
  const participants = Object.entries(boardSnap.get('participants') || {}).reduce((acc, [participantId, roleData]) => {
    if (participantId === boardSnap.get(CREATED_BY_PROP)) {
      // Do not modify board owner role because the owner will not see the board anymore.
      // (For example in "archived" folder)
      return { ...acc, [participantId]: roleData };
    }

    return {
      ...acc,
      [participantId]: assignUpdatedProps({
        data: {
          ...roleData,
          role: ROLE_REVOKED_VALUE,
        },
      }),
    };
  }, {});

  const data = Object.keys(participants).length > 0 ? { folder, participants } : { folder };
  return folder === FolderTypes.DELETED ? assignDeletedProps({ uid, data }) : data;
};

/**
 * Change participant role.
 *
 * @param {object} opts
 * @param {string} opts.participantUid
 * @param {number} opts.roleValue
 *
 * @returns {*}
 */
const changeParticipantRole = ({ participantUid, roleValue }) => ({
  [`participants.${participantUid}`]: assignUpdatedProps({ data: { role: roleValue } }),
});

module.exports = {
  setBoardFolder,
  changeParticipantRole,
};
