const test = require('tape');
const model = require('./index');

const { createSnap, createColRef, firestore } = require('../test')();

test('non existing', (t) => {
  t.throws(() => model(createSnap({}, 'foo/bar').ref).id()); //unknown type
  t.doesNotThrow(() => model(createSnap({}, 'profiles/test')).id());
  t.end();
});

test('boarditems empty', (t) => {
  const snap = createSnap({}, 'boards/123/items/321');
  const m = model(snap.ref);
  t.throws(() => m.create({})); //no title
  t.end();
});

test('boarditems', (t) => {
  const snap = createSnap({}, 'boards/123/items/321');
  const m = model(snap);
  t.equal(m.id(), '321');
  t.equal(m.itemId(), '321');
  t.equal(m.boardId(), '123');
  t.end();
});

test('boarditems by colref', (t) => {
  const col = createColRef('profiles/foo/notifications');
  const m = model(col);
  t.doesNotThrow(() => m.createBoardNotification({}));
  t.end();
});

test('boarditems by col path', (t) => {
  const m = model('profiles/foo/notifications', firestore());
  t.doesNotThrow(() => m.createBoardNotification({}));
  t.end();
});
