const models = require('../index');

/**
 * Create board role changed notification object.
 *
 * @param {object} opts
 * @param {string} opts.initiatorId
 * @param {string} opts.initiatorName
 * @param {FirebaseFirestore.DocumentSnapshot} opts.boardSnap
 * @param {FirebaseFirestore.CollectionReference} opts.notificationsRef
 * @param {number} opts.roleValue
 *
 * @returns {*}
 */
const createRoleChangedNotification = ({ initiatorId, initiatorName, boardSnap, notificationsRef, roleValue }) => {
  const data = {
    type: 'role',
    user: {
      uid: initiatorId,
      displayName: initiatorName,
    },
  };

  const meta = {
    board: {
      title: boardSnap.get('title'),
      id: boardSnap.id,
      role: roleValue,
    },
  };

  const { createBoardNotification } = models(notificationsRef);
  return createBoardNotification(data, meta);
};

/**
 * Create board reminder notification object.
 *
 * @param {object} opts
 * @param {object} opts.board
 * @param {FirebaseFirestore.CollectionReference} opts.notificationsRef
 *
 * @returns {object}
 */
const createBoardReminderNotification = ({ board, notificationsRef }) => {
  const data = {
    type: 'reminder',
    user: {
      uid: board.created_by,
    },
  };

  const meta = {
    board: {
      id: board.id,
      title: board.title,
    },
  };

  const { createBoardNotification } = models(notificationsRef);
  return createBoardNotification(data, meta);
};

module.exports = { createRoleChangedNotification, createBoardReminderNotification };
