const { assignDeletedProps } = require('../../common/utils');

/**
 * Set profile deleted properties.
 *
 * @param {FirebaseFirestore.DocumentSnapshot} profileSnap
 *
 * @returns {*}
 */
const setProfileDeleted = ({ profileSnap }) =>
  assignDeletedProps({
    data: {
      displayName: 'Verwijderde gebruiker',
      photoURL: null,
      public: false,
      created_at: profileSnap.get('created_at') || null,
      _copernica_id: profileSnap.get('_copernica_id') || null,
    },
  });

/**
 * Get user display name.
 */
const getDisplayName = (user) => {
  let { displayName } = user || {};

  if (!displayName && typeof user.email === 'string') displayName = user.email.substring(0, user.email.indexOf('@'));

  if (!displayName) displayName = 'Gebruiker';

  return displayName;
};

/**
 * Get profile object with default properties.
 */
const getDefaultProfile = (user) => ({
  created_by: user.uid,
  created_at: new Date(),
  updated_at: new Date(),
  public: true,
  displayName: getDisplayName(user),
  photoURL: user.photoURL || null,
});

module.exports = { setProfileDeleted, getDisplayName, getDefaultProfile };
