/* eslint-disable camelcase */
const { throwIfMissingProps } = require('../helpers');

module.exports = (info = {}) => {
  const { snap } = info;

  if (!snap) {
    return {
      createSuggestionActivity: (props) => {
        throwIfMissingProps(['uid', 'urlref.id', 'url'], props);

        const { uid, urlref, public: public_a, parent_activity_id, displayName, url } = props;

        const act = {
          created_at: new Date(),
          updated_at: new Date(),
          created_by: uid,
          type: 'suggestion',
          user: {
            uid,
            displayName: displayName || 'gebruiker',
          },
          public: typeof public_a === 'undefined' ? true : !!public_a,
          scrape: {
            id: urlref.id,
            url,
          },
        };

        if (urlref.product) {
          act.scrape.product = {};
          if (urlref.product.title) {
            act.scrape.product.title = urlref.product.title;
          }

          if (urlref.product.price) {
            act.scrape.product.price = urlref.product.price;
          }
        }

        if (parent_activity_id) {
          act.parent_activity_id = parent_activity_id;
        }

        return act;
      },
      createProductChangeActivity: (props) => {
        throwIfMissingProps(['item_id', 'item_title', 'price.before', 'price.after'], props);

        const {
          created_at: given_created_at,
          item_id,
          item_title,
          price: { before, after, currency },
        } = props;

        const act = {
          created_at: given_created_at || new Date(),
          type: 'productUpdate',
          item: {
            id: item_id,
            title: item_title,
            changes: {
              price: {
                before,
                after,
                currency,
              },
            },
          },
        };
        return act;
      },
    };
  }

  return {};
};
