const { InviteStatus } = require('../../schemas/firestore/objects/profiles/actions/invite-status');

module.exports = (info) => {
  const {
    snap,
    // type,
    // ref,
    // isCollection
  } = info;

  const createNotification = (type, data, meta) => ({
    type,
    data,
    read: false,
    created_at: new Date(),
    ...(meta ? { meta } : {}),
  });

  if (!snap) {
    return {
      createEventNotification: (data, meta) => createNotification('event', data, meta),
      createBoardNotification: (data, meta) => createNotification('board', data, meta),
      createFollowNotification: (data) => createNotification('follow', data),

      /**
       * Create a notification object based on the profile action object.
       *
       * @param {string} inviteStatus
       * @param {object} data
       *
       * @returns {{read: boolean, data, meta: *, created_at: Date, type}}
       */
      createShareProductNotification: ({ inviteStatus, data }) => {
        switch (inviteStatus) {
          case InviteStatus.Sent:
            return createNotification('shared_product', data);
          case InviteStatus.Accepted:
            return createNotification('shared_product_accepted', data);
          case InviteStatus.Rejected:
            return createNotification('shared_product_rejected', data);
          default:
            throw new Error(`Invite status is not supported - '${inviteStatus}'`);
        }
      },
    };
  }

  return {};
};
