const test = require('tape');
const { firestore, makeChange } = require('firebase-functions-test')();
const {
  refDoc,
  // addRef,
  // removeRef,
  // setRefs,
  // populateRefs,
  // updateRefCount,
  // updateSubColCount
  getRefsDelta,
} = require('./index');

test('refDoc', (t) => {
  const docSnap1 = firestore.makeDocumentSnapshot({ foo: 'bar' }, 'foo/bar');
  const docSnap2 = firestore.makeDocumentSnapshot({ bax: 'bax' }, 'bax/baz');

  t.equal(refDoc(docSnap1, docSnap2.ref).path, 'foo/bar/refs/bax_baz');
  t.equal(refDoc('foo/bar', docSnap2.ref).path, 'foo/bar/refs/bax_baz');

  t.end();
});

test('getRefsDelta add', (t) => {
  const ref1 = firestore.makeDocumentSnapshot({ ding: 'dong' }, 'bliep/bloep').ref;

  const docSnap1 = firestore.makeDocumentSnapshot({ foo: 'bar' }, 'foo/bar');
  const docSnap2 = firestore.makeDocumentSnapshot({ foo: 'bar', ref1 }, 'foo/bar');

  const change = makeChange(docSnap1, docSnap2);

  const refsDelta = getRefsDelta(change);

  t.equal(refsDelta.refsToBeAdded.length, 1);
  t.equal(refsDelta.refsToBeDeleted.length, 0);
  t.equal(refsDelta.oldrefs.length, 0);
  t.equal(refsDelta.newrefs.length, 1);

  t.end();
});

test('getRefsDelta add omitPaths', (t) => {
  const ref1 = firestore.makeDocumentSnapshot({ ding: 'dong' }, 'bliep/bloep').ref;

  const docSnap1 = firestore.makeDocumentSnapshot({ foo: 'bar' }, 'foo/bar');
  const docSnap2 = firestore.makeDocumentSnapshot({ foo: 'bar', ref1 }, 'foo/bar');

  const change = makeChange(docSnap1, docSnap2);

  const refsDelta = getRefsDelta(change, { omitPaths: ['ref1'] });

  t.equal(refsDelta.refsToBeAdded.length, 0);
  t.equal(refsDelta.refsToBeDeleted.length, 0);
  t.equal(refsDelta.oldrefs.length, 0);
  t.equal(refsDelta.newrefs.length, 0);

  t.end();
});

test('getRefsDelta add/remove', (t) => {
  const ref1 = firestore.makeDocumentSnapshot({ ding: 'dong' }, 'bliep/bloep1').ref;
  const ref2 = firestore.makeDocumentSnapshot({ ding: 'dong' }, 'bliep/bloep2').ref;
  const ref3 = firestore.makeDocumentSnapshot({ ding: 'dong' }, 'bliep/bloep3').ref;

  const docSnap1 = firestore.makeDocumentSnapshot({ foo: 'bar', ref1, ref3 }, 'foo/bar');
  const docSnap2 = firestore.makeDocumentSnapshot({ foo: 'bar', ref2, ref3 }, 'foo/bar');

  const change = makeChange(docSnap1, docSnap2);

  const refsDelta = getRefsDelta(change);

  t.equal(refsDelta.refsToBeAdded.length, 2);
  t.equal(refsDelta.refsToBeDeleted.length, 1);
  t.equal(refsDelta.oldrefs.length, 2);
  t.equal(refsDelta.newrefs.length, 2);

  t.end();
});

test('getRefsDelta add/remove alwaysAdd:false', (t) => {
  const ref1 = firestore.makeDocumentSnapshot({ ding: 'dong' }, 'bliep/bloep1').ref;
  const ref2 = firestore.makeDocumentSnapshot({ ding: 'dong' }, 'bliep/bloep2').ref;
  const ref3 = firestore.makeDocumentSnapshot({ ding: 'dong' }, 'bliep/bloep3').ref;

  const docSnap1 = firestore.makeDocumentSnapshot({ foo: 'bar', ref1, ref3 }, 'foo/bar');
  const docSnap2 = firestore.makeDocumentSnapshot({ foo: 'bar', ref2, ref3 }, 'foo/bar');

  const change = makeChange(docSnap1, docSnap2);

  const refsDelta = getRefsDelta(change, { alwaysAdd: false });

  t.equal(refsDelta.refsToBeAdded.length, 1);
  t.equal(refsDelta.refsToBeDeleted.length, 1);
  t.equal(refsDelta.oldrefs.length, 2);
  t.equal(refsDelta.newrefs.length, 2);

  t.end();
});

test('getRefsDelta add/remove always add', (t) => {
  const ref1 = firestore.makeDocumentSnapshot({ ding: 'dong' }, 'bliep/bloep1').ref;
  const ref2 = firestore.makeDocumentSnapshot({ ding: 'dong' }, 'bliep/bloep2').ref;

  const docSnap1 = firestore.makeDocumentSnapshot({ foo: 'bar', ref1 }, 'foo/bar');
  const docSnap2 = firestore.makeDocumentSnapshot({ foo: 'bar', ref2 }, 'foo/bar');

  const change = makeChange(docSnap1, docSnap2);

  const refsDelta = getRefsDelta(change);

  t.equal(refsDelta.refsToBeAdded.length, 1);
  t.equal(refsDelta.refsToBeDeleted.length, 1);
  t.equal(refsDelta.oldrefs.length, 1);
  t.equal(refsDelta.newrefs.length, 1);

  t.end();
});
