const mergeWith = require('lodash/mergeWith');

/**
 * Special merge function that will concatenate arrays for clashing keys. Must be used in combination with lodash
 * function `mergeWith`.
 *
 * See: https://lodash.com/docs/4.17.10#mergeWith
 *
 * Example:
 *
 * console.log( mergeWith({a:['b']}, {a:['c']}, concatArrays) )
 *
 * // {a:['b', 'c']}
 *
 * @param objValue
 * @param srcValue
 * @returns {any[]}
 */
const concatArrays = (objValue, srcValue) => {
  if (Array.isArray(objValue)) {
    return Array.from(new Set(objValue.concat(srcValue)));
  }
};

/**
 * Simple inheritance for json schema since it is not supported yet.
 * https://json-schema.org/understanding-json-schema/reference/combining.html#allof (see last paragraph)
 *
 * You can not combine schemas while preserving "additionalProperties: false"
 */
const inherit = (...args) => mergeWith({}, ...args, concatArrays);

module.exports = {
  inherit,
};
