/* eslint-disable camelcase */

import { FolderType } from '../../../common/boards/folder-types';
import { Schema } from '../../schema';
import { Editable } from './editable';
import { Poster } from './poster';
import { User } from './user';

export type BoardParticipant = {
  role: number;
  updated_at: Date;
};

export type BoardParticipants = {
  [uid: string]: BoardParticipant;
};

type BoardSort = 'az' | 'price' | 'created' | 'updated' | 'custom';

export type BoardGroup = {
  name: string;
  slug: string;
  items: string[];
};

export type Board = Omit<Editable, 'created_by'> & {
  created_by: string;
  title: string;
  type?: 'checkoff';
  auth?: {
    question?: string;
    password?: string;
  };
  participants: BoardParticipants;
  cloned_from?: string;
  folder?: FolderType;
  budget?: number;
  budget_seperate?: boolean | 'item_budget' | 'true' | 'total_budget' | 'total';
  default_board?: boolean;
  view?: 'list' | 'masonry';
  description?: string | null;
  public?: boolean;
  _temp_cat_label?: string | null;
  _temp_rating?: number;
  _slug?: string;
  tags?: string[];
  sort?: {
    list?: BoardSort;
    masonry?: BoardSort;
  };
  hidden_fulfilled?: boolean;
  user?: User;
  poster?: Poster | null;
  remind_at?: Date | null;
  groups?: BoardGroup[];
};

declare const BoardSchema: Schema;

export default BoardSchema;
