const { inherit } = require('../../../schema');
const editable = require('./editable');

module.exports = inherit(editable, {
  $id: 'http://hopponn.com/schemas/board.json',
  definitions: {
    sort: {
      type: 'string',
      enum: ['az', 'price', 'created', 'updated', 'custom'],
    },
  },
  required: [
    'created_by',
    'title',
    'participants',
    // "public" // TODO make required some day
  ],
  properties: {
    type: { type: 'string', enum: ['checkoff'] },
    auth: {
      type: 'object',
      properties: {
        question: { type: 'string' },
        password: { type: 'string' },
      },
    },
    cloned_from: {
      type: 'string',
    },
    folder: { $ref: 'http://hopponn.com/schemas/folder.json' },
    budget: {
      type: 'number',
      default: 0.0,
    },
    budget_seperate: {
      enum: [
        'item_budget',
        true, // 'item_budget', true = same
        'total_budget',
        false, // 'total_budget', false, = same
        'total',
      ],
    },
    default_board: {
      type: 'boolean',
    },
    title: { type: 'string' },
    view: { type: 'string', enum: ['list', 'masonry'] },
    description: {
      oneOf: [{ type: 'string' }, { type: 'null' }],
    },
    public: { type: 'boolean' },
    _temp_cat_label: {
      oneOf: [{ type: 'string' }, { type: 'null' }],
    },
    _temp_rating: { type: 'number' },
    tags: {
      type: 'array',
      items: {
        type: 'string',
      },
    },
    sort: {
      type: 'object',
      additionalProperties: false,
      properties: {
        list: {
          oneOf: [
            {
              type: 'object',
              additionalProperties: false,
              required: ['direction', 'field'],
              properties: {
                direction: { type: 'string' },
                field: { type: 'string' },
              },
            },
            { $ref: '#/definitions/sort' },
          ],
        },
        masonry: { $ref: '#/definitions/sort' },
      },
    },
    hidden_fulfilled: { type: 'boolean' },
    user: { $ref: 'http://hopponn.com/schemas/user.json' },
    participants: { $ref: 'http://hopponn.com/schemas/participants.json' },
    poster: { $ref: 'http://hopponn.com/schemas/poster.json' },
    _slug: { type: 'string' },
    remind_at: {
      oneOf: [{ $ref: 'http://hopponn.com/schemas/date.json' }, { type: 'null' }],
    },
    column_config: {
      type: 'array',
      items: {
        type: 'object',
        additionalProperties: false,
        required: ['field', 'isVisible'],
        properties: {
          field: { type: 'string' },
          isVisible: { type: 'boolean' },
        },
      },
    },
    custom_columns: {
      type: 'array',
      items: {
        type: 'object',
        additionalProperties: false,
        required: ['field', 'fieldType', 'label'],
        properties: {
          field: { type: 'string' },
          fieldType: { type: 'string' },
          label: { type: 'string' },
        },
      },
    },
  },
});
