/**
 * BOARD ITEM CORE - All the properties that make up a board item in any mode.
 */
const { inherit } = require('../../../../../schema');
const editable = require('../../editable');
const gui = require('./gui');

module.exports = inherit(editable, gui, {
  $id: 'http://hopponn.com/schemas/boards/items/core.json',
  required: ['mode'],
  properties: {
    title: { type: 'string' }, // custom title, if empty "urlref.product.title" or "urlref.content.title" will be used
    notes: { type: 'string' }, // custom notes
    include: { type: 'boolean' }, // whether to include in budget calculations
    amount: { type: 'integer' }, // amount for budget calculations
    rescrape_until: { oneOf: [{ $ref: 'http://hopponn.com/schemas/date.json' }, { type: 'null' }] },
    preview: { type: 'null' }, // temporary visual placeholder while assets are beging retrieved
    assets: { $ref: 'assets.json' }, // custom assets, apart from the urlref assets
    main_asset: { type: 'string' }, // TODO: check if value references an id in "assets" or "urlref.assets"
    description: { oneOf: [{ type: 'string' }, { type: 'null' }] }, // custom description, if none found in "urlref.product.description"
    price: { oneOf: [{ type: 'number' }, { type: 'null' }] },
    product: {
      description: 'Snapshot of the product as it was when plinking the url',
      oneOf: [
        {
          $ref: 'http://hopponn.com/schemas/product.json',
        },
        {
          type: 'null',
        },
      ],
    },
    offerer: {
      type: 'object',
      required: ['domain', 'name'],
      properties: {
        domain: { type: 'string' },
        name: { type: 'string' },
        logo: { type: 'string' },
      },
    },
    url: {
      type: 'string',
      description:
        'This is where the url is stored. The scrape mechanism uses this property to execute the actual scrape',
    },
    _num_likes: { type: 'integer' },
    fulfilled: {
      type: 'object',
      patternProperties: {
        '.+': {
          type: 'object',
          additionalProperties: false,
          required: ['created_at'],
          properties: {
            guest: { type: 'boolean' },
            created_at: { $ref: 'http://hopponn.com/schemas/date.json' },
          },
        },
      },
      propertyNames: {
        type: 'string',
      },
    },
    custom_columns: {
      type: 'object',
      patternProperties: {
        '.+': {
          oneOf: [
            {
              type: 'object',
              additionalProperties: false,
              required: ['amount', 'currency'],
              properties: {
                amount: { type: 'number' },
                currency: { type: 'string' },
              },
            },
            {
              $ref: 'http://hopponn.com/schemas/date.json',
            },
            {
              type: 'number',
            },
            {
              type: 'boolean',
            },
            {
              type: 'string',
            },
            {
              type: 'null',
            },
          ],
        },
      },
      propertyNames: {
        type: 'string',
      },
    },
  },
});
