module.exports = {
  $id: 'http://hopponn.com/schemas/cloudinary.json',
  type: 'object',
  definitions: {
    url: {
      type: 'string',
      // "format": "url_rel_proto"
    },
    moderation: {
      type: 'object',
      additionalProperties: false,
      properties: {
        kind: {
          type: 'string',
          enum: ['aws_rek'],
        },
        status: {
          type: 'string',
          enum: ['approved', 'rejected', 'pending'],
        },
        updated_at: {
          type: 'string',
          format: 'date-time',
        },
        response: {
          type: 'object',
          additionalProperties: false,
          properties: {
            moderation_labels: {
              type: 'array',
              items: {
                type: 'object',
                additionalProperties: false,
                properties: {
                  confidence: {
                    type: 'number',
                  },
                  name: {
                    type: 'string',
                  },
                  parent_name: {
                    type: 'string',
                  },
                },
              },
            },
          },
        },
      },
    },
  },
  anyOf: [
    {
      // "additionalProperties": false,
      required: ['error'],
      properties: {
        error: { $ref: 'http://hopponn.com/schemas/cloudinary/error.json' },
        params: { $ref: 'http://hopponn.com/schemas/cloudinary/params.json' },
        info: { $ref: 'http://hopponn.com/schemas/cloudinary/info.json' },
        predominant: { $ref: 'http://hopponn.com/schemas/cloudinary/predominant.json' },
        colors: {
          type: 'array',
          item: { $ref: 'http://hopponn.com/schemas/cloudinary/color.json' },
        },
        notification_type: { type: 'string', enum: ['error'] },
      },
    },
    {
      additionalProperties: false,
      required: ['public_id'],
      properties: {
        created_at: { type: 'string', format: 'date-time' },

        public_id: { type: 'string' },
        existing: { type: 'boolean' },
        pending: { type: 'boolean' },
        overwritten: { type: 'boolean' },
        resource_type: { type: 'string', enum: ['image'] },
        type: { type: 'string', enum: ['upload'] },
        placeholder: { type: 'boolean' },
        notification_type: { type: 'string', enum: ['upload', 'error'] }, // TODO: is this correct? shouldn't error be the state above?
        original_filename: { type: 'string' },
        original_extension: { type: 'string' },

        format: { type: 'string', enum: ['jpg', 'png', 'gif', 'svg', 'webp', 'bmp'] }, // TODO: stritcly seen: svg is forbidden
        bytes: { type: 'number' },
        etag: { type: 'string' },

        signature: { type: 'string' }, // TODO: specific for file uploads?
        upload: { type: 'object' }, // TODO: specific for file uploads?

        grayscale: { type: 'boolean' },
        height: { type: 'number' },
        width: { type: 'number' },
        illustration_score: { type: 'number', minimum: 0, maximum: 1 },
        phash: { type: 'string' },
        semi_transparent: { type: 'boolean' },

        timestamp: { type: 'string', format: 'date-time' },
        request_id: { type: 'string' },

        url: { $ref: '#/definitions/url' },
        secure_url: { $ref: '#/definitions/url' },
        version: { type: 'number' },

        params: { $ref: 'http://hopponn.com/schemas/cloudinary/params.json' },
        error: { $ref: 'http://hopponn.com/schemas/cloudinary/error.json' },

        tags: {
          type: 'array',
          items: { type: 'string' },
        },

        // Face detection
        coordinates: {
          type: 'object',
        },

        moderation: {
          oneOf: [
            { $ref: '#/definitions/moderation' },
            {
              type: 'array',
              items: { $ref: '#/definitions/moderation' },
            },
          ],
        },

        info: { $ref: 'http://hopponn.com/schemas/cloudinary/info.json' },
        predominant: { $ref: 'http://hopponn.com/schemas/cloudinary/predominant.json' },
        colors: {
          type: 'array',
          items: { $ref: 'http://hopponn.com/schemas/cloudinary/color.json' },
        },

        image_metadata: { type: 'object' },
        exif: { type: 'object' },

        context: {
          type: 'object',
          additionalProperties: false,
          properties: {
            custom: {
              type: 'object',
              additionalProperties: false,
              properties: {
                originalUrl: {
                  type: 'string',
                  // "format": "url_rel_proto"
                },
                originalFile: {
                  type: 'string',
                },
              },
            },
          },
        },
      },
    },
  ],
};
