/**
 * EDITABLE - Core elements of a document that is directly or indirectly modifiable by the users of our system.
 *
 * The purpose of this JSON schema is to make descendant schemas more readable by leaving out the very basic properties
 * for an editable document while still being able to utilize it.
 *
 * Example:
 *
 * const { inherit } = require('@utils/schema');
 * const editable = require('./editable');
 *
 * module.exports = inherit(editable, {
 *     "$id": "http://hopponn.com/schemas/yourInheritedSchema.json",
 *     "properties": {
 *         "yourExtraProp":{ "type":"string" }
 *     }
 * });
 *
 */
const { inherit } = require('../../../schema');
const doc = require('./doc');

module.exports = inherit(doc, {
  $id: 'http://hopponn.com/schemas/editable.json',
  properties: {
    created_by: { type: 'string' },
    created_at: { $ref: 'http://hopponn.com/schemas/date.json' },
    updated_at: { $ref: 'http://hopponn.com/schemas/date.json' },
    updated_by: { type: 'string' },
    deleted_at: { $ref: 'http://hopponn.com/schemas/date.json' },
    deleted_by: { type: 'string' },
  },
  required: ['created_at'],
});
