const { inherit } = require('../../../schema');
const editable = require('./editable');

module.exports = inherit(editable, {
  $id: 'http://hopponn.com/schemas/profile.json',
  type: 'object',
  additionalProperties: false,
  required: ['public', 'displayName', 'photoURL'],
  properties: {
    deleted_at: false,
    public: { type: 'boolean' },
    displayName: { type: 'string' },
    photoURL: { $ref: 'http://hopponn.com/schemas/profiles/photoURL.json' },

    about: { type: 'string' },
    birthdate: {
      oneOf: [{ $ref: 'date.json' }, { type: 'null' }],
    },
    gender: {
      oneOf: [
        {
          type: 'string',
          enum: ['male', 'female'],
        },
        {
          type: 'null',
        },
      ],
    },
    assets: { $ref: 'assets.json' },
    interests: { type: 'array', items: { type: 'string' } },
    managed_domains: { type: 'array', items: { type: 'string' } },

    locale: { type: 'string' },

    push_disabled: {
      type: 'object',
      additionalProperties: false,
      properties: {
        follow: { type: 'boolean' },
        board_like: { type: 'boolean' },
        board_fulfilled: { type: 'boolean' },
        board_create: { type: 'boolean' },
        board_joined: { type: 'boolean' },
        board_board: { type: 'boolean' },
        board_comment: { type: 'boolean' },
        board_suggestion: { type: 'boolean' },
        board_productUpdate: { type: 'boolean' },
      },
    },
    _slug: { type: 'string' },
    _copernica_id: { type: 'string' },
    _stripe_customer_id: { type: 'string' },
    _optins: {
      // TODO: To be removed. Email Optins are handled at Copernica side.
      type: 'object',
    },
    _referral: {
      type: 'object',
    },
    _num_followers: { type: 'number' },
    _num_following: { type: 'number' },

    _num_boards: { type: 'number' },
    _num_boards_public: { type: 'number' },
    _num_public_boards_with_assets: { type: 'number' },

    _has_items: { type: 'boolean' },
    _has_invites: { type: 'boolean' },
    _has_boards: { type: 'boolean' },
    _app: {
      type: 'array',
      items: {
        enum: ['ios', 'android', 'web', 'browser-extension'],
      },
    },
    _account_type: { type: 'string' },
    _temp_rating: { type: 'number', titoe: 'Our score boost to make users more popular' },
    _rating: { type: 'number', title: 'Hatsa review rating' },
    social: {
      type: 'object',
      additionalProperties: false,
      properties: {
        instagram: { type: 'string' },
        youtube: { type: 'string' },
        facebook: { type: 'string' },
        twitter: { type: 'string' },
        blog: { type: 'string' },
      },
    },
  },
});
