const { inherit } = require('../../../../../../schema');
const editable = require('../../../editable');
const { InviteStatus } = require('../invite-status');
const { SharedProductSchemaId } = require('./shared-product');
const { getSchemaId } = require('../../../../../../validate/ajv');

const ShareProductActionSchemaId = getSchemaId('profiles/actions/shareProductDoc');

const ShareProductActionType = 'share_product';

const ProductType = Object.freeze({
  Product: 'product',
  UrlRef: 'urlref',
});

const ShareProductActionSchema = inherit(editable, {
  $id: ShareProductActionSchemaId,
  required: ['type', 'data'],
  additionalProperties: false,
  properties: {
    type: { const: ShareProductActionType },
    data: {
      type: 'object',
      required: ['product', 'invite', 'inviter', 'type'],
      additionalProperties: false,
      properties: {
        type: { enum: Object.values(ProductType) },
        product: { $ref: SharedProductSchemaId },
        invite: {
          type: 'object',
          required: ['status'],
          properties: {
            status: { enum: Object.values(InviteStatus) },
            message: { oneOf: [{ type: 'string' }, { type: 'null' }] },
          },
        },
        inviter: {
          type: 'object',
          required: ['uid'],
          properties: {
            uid: { type: 'string', minLength: 5 },
          },
        },
      },
    },
  },
});

module.exports = {
  ShareProductActionType,
  ProductType,
  ShareProductActionSchemaId,
  ShareProductActionSchema,
};
