const { inherit } = require('../../../schema');
const editable = require('./editable');

module.exports = inherit(editable, {
  $id: 'http://hopponn.com/schemas/urlref.json',
  type: 'object',
  definitions: {
    url: {
      type: 'string',
      format: 'url_rel_proto',
    },
    domain: {
      type: 'string',
      format: 'domain',
    },
  },
  required: [
    // "product",
    // "content",
    'urls',
    'domains',
  ],
  additionalProperties: false,
  properties: {
    dead: { type: 'boolean' },
    refcount: { type: 'number' },
    scrapecount: { type: 'number' },
    version: { type: 'string' },
    refversion: { type: 'number' }, // TODO: do we still need this?

    _temp_rating: { type: 'number' },

    content: {
      type: 'object',
      additionalProperties: false,
      properties: {
        images: {
          type: 'array',
          // "items": { "$ref": "#/definitions/url" }
        },
        videos: {
          type: 'array',
          items: { $ref: '#/definitions/url' },
        },
        keywords: {
          type: 'array',
          items: { type: 'string' },
        },
        titles: {
          type: 'array',
          items: { type: 'string' },
        },
        description: { type: 'string' },
      },
    },

    meta: {
      type: 'object',
      properties: {
        lang: { type: 'string' }, // TODO: format: lagnuage code?
        type: { type: 'string' },
      },
    },

    urls: {
      type: 'object',
      additionalProperties: false,
      properties: {
        canonical: { $ref: '#/definitions/url' },
        given: { $ref: '#/definitions/url' },
        normalized: { $ref: '#/definitions/url' },
      },
    },

    product: {
      oneOf: [{ $ref: 'product.json' }, { type: 'null' }],
    },

    image: { $ref: '#/definitions/url' },
    additional_images: {
      type: 'array',
      items: { $ref: '#/definitions/url' },
    },

    domains: {
      type: 'array',
      items: { $ref: '#/definitions/domain' },
    },

    offerer: {
      type: 'object',
      additionalProperties: false,
      properties: {
        name: { type: 'string' },
        domain: { $ref: '#/definitions/domain' },
        logo: { $ref: '#/definitions/url' },
      },
    },

    assets: {
      type: 'object',
      additionalProperties: false,
      properties: {
        'product.image': { $ref: 'assets.json' },
        'product.additional_images': { $ref: 'assets.json' },
        'content.images': { $ref: 'assets.json' },
      },
    },
  },
});
