const ShopAssetSchema = {
  type: 'object',
  required: ['publicId'],
  properties: {
    publicId: { type: 'string' },
    version: { type: 'number' },
    dominantColors: { type: 'array', items: { type: 'string' } },
    updated_at: { $ref: 'http://hopponn.com/schemas/date.json' },
  },
};

module.exports = {
  $id: 'http://hopponn.com/schemas/offerersDoc.json',
  type: 'object',
  additionalProperties: false,
  properties: {
    name: { type: 'string' },
    domain: { type: 'string' }, // TODO: type domain?
    description: { type: 'string' },
    logo: { type: 'string' },
    banner: { type: 'string' },
    usps: { type: 'array', items: { type: 'string' } },
    categories: { type: 'array', items: { type: 'string' } },
    assets: {
      type: 'object',
      properties: {
        logo: ShopAssetSchema,
        banner: ShopAssetSchema,
      },
    },
    blacklist: { type: 'boolean' },
    _temp_rating: { type: 'number' },
    domains: {
      type: 'array',
      items: { type: 'string' },
    },
    tags: {
      type: 'array',
      items: { type: 'string' },
    },
    affiliation: {
      type: 'object',
      additionalProperties: false,
      properties: {
        useWWW: { type: 'boolean' },
        trailingSlash: { type: 'boolean' },
        last_updated: { $ref: 'http://hopponn.com/schemas/date.json' },
        identification: {
          type: 'object',
          additionalProperties: false,
          properties: {
            product: {
              type: 'object',
              additionalProperties: false,
              properties: {
                regex: { type: 'string' }, // TODO: type regex?
                params: {
                  type: 'array',
                  items: {
                    type: 'object',
                    patternProperties: {
                      '.+': { type: 'string' },
                    },
                    propertyNames: { type: 'string' },
                  },
                },
              },
            },
          },
        },
        mutation: {
          type: 'object',
          additionalProperties: false,
          properties: {
            type: { type: 'string' },
            parameters: {
              type: 'object',
              patternProperties: {
                '.+': {
                  oneOf: [{ type: 'string' }, { type: 'integer' }],
                },
              },
              propertyNames: { type: 'string' },
            },
          },
        },
      },
    },
    products_priority: { type: 'number' },
    settings: {
      type: 'object',
      properties: {
        scraper: {
          type: 'object',
          properties: {
            scraper: {
              type: 'object',
              properties: {
                type: { type: 'string' }, // TODO: enum?
                proxy: { type: 'boolean' },
              },
            },
          },
        },
      },
    },
    draft: {
      type: 'object',
      properties: {
        name: { type: 'string' },
        domain: { type: 'string' },
        description: { type: 'string' },
        usps: { type: 'array', items: { type: 'string' } },
        categories: { type: 'array', items: { type: 'string' } },
        assets: {
          type: 'object',
          properties: {
            logo: ShopAssetSchema,
            banner: ShopAssetSchema,
          },
        },
      },
      required: ['name', 'domain'],
    },
    managed_by: {
      type: 'object',
      patternProperties: {
        '.+': {
          type: 'object',
          required: ['role'],
          properties: {
            role: { enum: [1000, 2000, 3000] },
            updated_at: { $ref: 'http://hopponn.com/schemas/date.json' },
          },
        },
      },
    },
    promotions: {
      type: 'object',
      patternProperties: {
        '.+': {
          type: 'object',
          required: ['name', 'valid_from', 'valid_to', 'link'],
          properties: {
            id: { type: 'string' },
            name: { type: 'string' },
            description: { type: 'string' },
            valid_from: { $ref: 'http://hopponn.com/schemas/date.json' },
            valid_to: { $ref: 'http://hopponn.com/schemas/date.json' },
            code: { type: 'string' },
            link: { type: 'string' },
            updated_by: { type: 'string' },
            updated_at: { $ref: 'http://hopponn.com/schemas/date.json' },
          },
        },
      },
    },
    highlighted_boards: {
      type: 'object',
      patternProperties: {
        '.+': {
          type: 'object',
          required: ['_slug'],
          properties: {
            _slug: { type: 'string' },
            updated_by: { type: 'string' },
            updated_at: { $ref: 'http://hopponn.com/schemas/date.json' },
          },
        },
      },
    },
    highlighted_products: {
      type: 'object',
      patternProperties: {
        '.+': {
          type: 'object',
          properties: {
            text: { type: 'string' },
          },
        },
      },
    },
    verification: {
      type: 'object',
      properties: {
        code: { type: 'string' },
        verified: { type: 'boolean' },
      },
    },
    parentId: {
      oneOf: [{ type: 'string' }, { type: 'null' }],
    },
    checkFields: {
      type: 'object',
      properties: {
        parentId: { $ref: 'http://hopponn.com/schemas/checkField.json' },
      },
    },
    affiliateProgram: {
      type: 'string',
    },
    aggregated_metadata: {
      first: { type: 'date' },
      last: { type: 'date' },
      item_count: { type: 'number' },
      product_count: { type: 'number' },
      profile_count: { type: 'number' },
    },
    subscription: {
      type: 'object',
      properties: {
        id: { type: 'string' },
        type: {
          type: 'string',
          enum: ['basic', 'pro', 'plus'],
        },
        current_period_end: { $ref: 'http://hopponn.com/schemas/date.json' },
      },
      required: ['id', 'current_period_end'],
    },
    updated_by: { type: 'string' },
    updated_at: { $ref: 'http://hopponn.com/schemas/date.json' },
  },
};
