const { getSchemaId } = require('../../../validate/ajv');
const photoURL = require('../../firestore/objects/profiles/photoURL');

/**
 * Base schema for notifications DTOs.
 */
module.exports = {
  $id: getSchemaId('notification'),
  type: 'object',
  additionalProperties: false,
  properties: {
    transport: { type: 'array', items: [{ enum: ['push', 'email'] }], minItems: 1 }, // Defines how to send notification, by push or email
    type: {
      type: 'string',
      enum: [
        'board_create',
        'board_like',
        'board_activity_like',
        'board_joined',
        'board_fulfilled',
        'board_board',
        'board_comment',
        'board_suggestion',
        'board_reminder',
        'follow',
        'board_productUpdate',
        'shared_product',
        'shared_product_accepted',
        'shared_product_rejected',
        'board_invite',
        'board_role',
        'board_left',
        'product_fulfilled_delivery',
        'product_fulfilled_trial',
        'event_participate',
        'event_name_drawn',
        'event_not_applied',
        'event_not_drawn',
        'event_board_empty',
        'event_over',
      ],
    },
    id: { oneOf: [{ type: 'string' }, { type: 'null' }] }, // Notification ID
    title: { type: 'string', minLength: 2 }, // Title of the message
    body: { type: 'string' }, // Body of the message
    profileId: { type: 'string', minLength: 5 }, // ID of the users profile
    subjectId: { type: 'string', minLength: 5 }, // ID of the "context" entity
    displayName: { type: 'string' }, // Users display name
    path: { type: 'string', minLength: 5 }, // Path to open when clicked on notification
    createdAt: { type: 'string', format: 'date-time' }, // Created at date in string format
    photoURL,
    badge: { type: 'number' },
    options: {
      type: 'object',
      additionalProperties: false,
      properties: {
        refreshBadge: {
          // Use true if you want to refresh badge with unread notifications count on handler part
          type: 'boolean',
        },
      },
    },
  },
  required: ['transport', 'type', 'title', 'body', 'profileId', 'subjectId', 'createdAt'],
};
