const { Writable } = require('stream');
const Queue = require('better-queue');

const genBatch = ({ handleBatch, batchSize, id, merge, filter, batchDelay, cb }) => {
  let total = 0;
  let deduped = 0;

  const qOpts = {
    batchSize: batchSize || 250,
    id: function (doc, cb) {
      total++;
      cb(null, id ? id(doc) : doc.id);
    },
    merge: function mergeTasksFunc(oldTask, newTask, cb) {
      // console.log(`dupe ${oldTask.id}`);
      deduped++;
      cb(null, merge ? merge(oldTask, newTask) : oldTask);
    },
    batchDelay: batchDelay || 5000, //wait at least 500 ms before starting a batch, handy for dedupes
    // maxRetries: 1,
    // retryDelay: 1000,
    // maxTimeout: 2000, //After 2 seconds, the process will throw an error instead of waiting for the callback to finish.
    // afterProcessDelay: 1000, //You can also set afterProcessDelay, which will delay processing between tasks.

    //You can define a function called precondition that checks that it's ok to process the next batch. If the preconditions fail, it will keep calling this function until it passes again.
    // precondition: function (cb) {}
    // preconditionRetryTimeout: 10*1000
  };

  function qHandler(batch, cb) {
    Promise.resolve(handleBatch(batch))
      .then((_resp) => {
        // console.log(`-- batch complete with ${batch.length} items`);
        cb(null, batch);
      })
      .catch((e) => {
        // console.log(e.message);
        cb(e);
      });
  }

  const queue = new Queue(qHandler, qOpts);

  // queue.on('task_queued', (taskId, taskData) => console.log('task_queued', taskId));
  // queue.on('task_accepted', (taskId, taskData) => console.log('task_accepted', taskId));
  // queue.on('task_started', (taskId, taskData) => console.log('task_started', taskId));
  // queue.on('task_finish', (taskId, error, stats) => console.log('task_finish', taskId, stats));
  // queue.on('task_failed', (...args) => console.log('task_failed', ...args));
  // queue.on('task_progress', (...args) => console.log('task_progress', ...args));

  queue.on('batch_finish', (result) => {
    const {
      // successRate,
      // peak,
      // average,
      total: batch_processed_total,
    } = queue.getStats();

    cb &&
      cb({
        // peak,
        // average,
        batch_processed_total,
        ingest_total: total,
        current_batch_deduped: deduped,
        current_batch_length: result.length,
      });
    // queue.resetStats();
  });
  queue.on('batch_progress', (...args) => console.log('batch_progress', ...args));
  queue.on('batch_failed', (err) => console.log('batch_failed', err));

  queue.on('empty', (...args) => console.log('empty', ...args));
  queue.on('drain', (...args) => console.log('drain', ...args));

  const ws = Writable({ objectMode: true });
  ws._write = function (chunk, enc, next) {
    if (!filter || filter(chunk)) queue.push(chunk);

    next();
  };

  return ws;
};

module.exports = genBatch;
