const fbtest = require('firebase-functions-test');

const genFsTestHelpers = (opts) => {
  const firebase = fbtest(opts);

  const { makeChange } = firebase;
  const { makeDocumentSnapshot } = firebase.firestore;

  const createSnap = (d, path = 'foo/bar') => makeDocumentSnapshot(d, path);
  const createChange = (a, b, path = 'foo/bar') => makeChange(createSnap(a, path), createSnap(b, path));

  const createColRef = (path) => createSnap({}, path + '/foobarbaxbaz').ref.parent;
  const firestore = () => createSnap({}).ref.firestore;

  return {
    cleanup: firebase.cleanup,

    //core helpers
    makeDocumentSnapshot,
    makeChange,

    //own helpers
    firestore,
    createSnap,
    createChange,
    createColRef,
  };
};

const setGetRefStubsAndAssertions = (t, refStub, op, path, updateMap) =>
  refStub.callsFake((docRef) => {
    //Check Path
    const fPath = docRef.path || docRef.before.ref.path;
    t.equal(fPath, path, 'correct docPath');

    //Check Method
    const refMethods = ['delete', 'update', 'set', 'get', 'collection', 'isEqual', 'onSnapshot'];

    if (op && !refMethods.includes(op)) return t.fail(`setGetRefStubsAndAssertions called with wrong operator: ${op}`);

    const fMethods = refMethods.filter((m) => m !== op);

    const stubs = fMethods.reduce((acc, val) => {
      acc[val] = () => Promise.resolve(t.fail(`"${val}" should not be called on ref`));
      return acc;
    }, {});

    if (op === 'delete') stubs[op] = () => Promise.resolve(t.ok(`"${op}" called on ref`));

    if (op === 'set' || op === 'update')
      stubs[op] = (um /*, opts*/) => {
        return Promise.resolve(t.deepEqual(updateMap, um, `"${op}" called with correct updateMap`));
      };

    return stubs;
  });

module.exports = genFsTestHelpers;
module.exports.setGetRefStubsAndAssertions = setGetRefStubsAndAssertions;
